#!/usr/bin/env python
#	
#	Command_Classes: A module defining various 'widget'
#		  classes to include in tkCoLa commands.
#		  Part of the tkCoLa suite.
#		  See documentation for details of use.
#		  Version 1.0, August 16, 2000, 5-ish
#
#		JEMWorks Software, 2000
#		http://www.jasonya.com/jemworks
#		jemworks@jasonya.com
#
#	tkCoLa: Tkinter Command Language
#		A mini-language which provides a simplified
#		interface to Tk widgets and allows rapid
#		prototyping of command interfaces and other
#		GUIs in Python.	
#

from Tkinter import *
from Dialog import Dialog
from Command_Init import *

from Command_Options import *
import os
import fnmatch

# Error class for the command parser
class CommandParseError:
	def __init__(self, key, field, msg=None):
      		self.key = key		# keyword being parsed
		self.field = field	# field causing the problem
		self.msg = msg		# message about the error
	def __str__(self):
		return self
	def tell(self):
		msg = self.msg or ''
		print 'CommandParseError in keyword %s, field %s - %s' % (self.key, self.field, msg)

# Error class for command callbacks
class CommandCallError:
	def __init__(self, level, call, msg):
		self.level = level	# int: 1=Alert 2=Error 3=Critical
      		self.call = call	# what is the name of the callback?
		self.msg = msg		# what was the error?
	def __str__(self):
		return self
	def tell(self, where):
		if self.level == 1:
			errstr = 'CommandCallAlert'
		elif self.level == 2:
			errstr = 'CommandCallError'
		else:
			errstr = 'CommandCallCritical'
		print '%s: %s: %s - %s' % (errstr, where, self.call, self.msg)
		
# Command RunTime Error
class CommandRunError:
	def __init__(self, level=0, command=None, option=None, pyroutine='', msg=''):
		if not level:
			self.errstr = 'CommandRunAlert'
		else:
			self.errstr = 'CommandRunError'
		self.command = command
		self.option = option
		self.pyroutine = pyroutine
		self.msg = msg
	def __str__(self):
		return self
	def tell(self):
		print '%s:\tcommand title- %s' % (self.errstr,self.command.label)
		print '\t\t\toption- %s' % self.option.var_name
		print '\t\t\tpython method- %s' % self.pyroutine
		print '\t\t\t%s' % self.msg

# Record for command parse variable passing
class Command_Parse_Record:
	def __init__(self, command=None, option=None, command_list=[],
			group_range=[], component_dict={}, tk_config={},
			manager=None, mem=None, echo=None,
			command_prefix='self.parent',**kw):
		self.command = command
		self.option = option
		self.command_list = command_list
		self.group_range = group_range
		self.component_dict = component_dict
		self.tk_config = tk_config
		self.echo = echo
		self.mem = mem
		self.command_prefix = command_prefix
		
		self.manager = manager
		self.parent = manager.parent
	
	def clear_command(self):
		self.command = None
		self.option = None

#####################################################################
#
#	class Color Picker
#
#
#
# A widget that allows the user to pick a color from
# a predefined array of colors or type in their own
# color designation in an input prompt.

class Color_Picker:
	def __init__(self, color_map=None, x_box=10, y_box=10,
			outline=1, rows=15, show_current=1,
			name_box=1, show=1, frame=None,
			color=None, **kw):
		# color map is simply a list of
		# tk color names
		if not color_map:
			self.color_map = colors
		# size of the display boxes
		self.x_box = x_box
		self.y_box = y_box
		# outline size, 0 = no outline
		self.outline = outline
		# rows of colors on the picker
		self.rows = rows
		# show the current color
		self.show_current = show_current
		# display a name box for the color
		self.name_box = name_box
		
		self.picked_color = None
		
		if show and frame:
			self.make(frame=frame, color=color)
		
	def make(self, frame=None, color='black', **kw):
		self.number = len(self.color_map)
		self.width = self.x_box * self.rows
		self.cols = self.number / self.rows
		if divmod(self.number, self.rows):
			self.cols = self.cols + 1
		self.height = self.cols * self.y_box
		self.picked_color = color or 'black'
		
		self.picker = Canvas(frame, width=self.width, height=self.height)
		self.picker.pack(side=TOP)
		
		y = 0
		ny = 0
		col = 0
		for i in range(1, (self.cols + 1)):
			x = 0
			nx = 0
			ny = y + self.y_box
			for n in range(1, (self.rows + 1)):
				nx = x + self.x_box
				try:
					colors = self.color_map[col]
				except IndexError:
					break
				
				self.picker.create_rectangle(x, y, nx, ny,
							   fill=colors, outline='black', 
							   width=self.outline, tags=colors)
				self.picker.tag_bind(colors, '<Button-1>', (lambda event=None, 
						self=self, colors=colors:
						self.select(colors)))
				self.picker.tag_bind(colors, '<Enter>', (lambda event=None, 
						self=self, colors=colors:
						self.mouse_over(colors)))
				x = nx
				col = col + 1
				if col > self.number:
					break
			y = ny
		
		n_frame = Frame(frame)
		n_frame.pack(side=BOTTOM, expand=1, fill=X)
		
		if self.name_box:
			self.entry_box = Entry(n_frame, width=20)	
			self.entry_box.pack(side=LEFT)
			self.entry_box.insert(END, self.picked_color)
		else:
			self.entry_box = None
		if self.show_current:
			self.button = Button(n_frame, width=4, height=2,
						background=self.picked_color)
			self.button.pack(side=LEFT)
	
	def select(self, color, **kw):
		self.picked_color = color
		if self.name_box:
			self.entry_box.delete(0,END)
			self.entry_box.insert(END, self.picked_color)
		if self.show_current:
			self.button['background'] = self.picked_color
			self.button.update()
			
	def mouse_over(self, color, **kw):
		if self.show_current:
			self.button['background'] = color
			self.button.update()
		
	def get(self, **kw):
		if self.name_box:
			return self.entry_box.get()
			
		return self.picked_color
