#!/usr/bin/env python
#	
#	Command_Options: A module designating
#		  tkCoLa Command Options.
#		  Part of the tkCoLa suite.
#		  See documentation for details of use.
#		  Version 1.0, August 16, 2000, 5-ish
#
#		JEMWorks Software, 2000
#		http://www.jasonya.com/jemworks
#		jemworks@jasonya.com
#
#	tkCoLa: Tkinter Command Language
#		A mini-language which provides a simplified
#		interface to Tk widgets and allows rapid
#		prototyping of command interfaces and other
#		GUIs in Python.	
#
import string
import os
import sys
import copy
import tkFont
import fnmatch

try:
	import Tkinter
	from Tkinter import *
except ImportError:
	print "Commands Error:"
	print "\ttkCoLa requires Tk/Tcl 8.0."
	print "\tSorry!"
	sys.exit(0)
	
try:
	from Command_Init import *
	from Command_Classes import *
	from Command_Functions import *
except ImportError, module:
	print "Commands Error:"
	print "\tCan't find %s module." % module
	sys.exit(0)

from types import *

class Option:
	def __init__(self, type=None, var_name=None, label=None, 
			var_type=None, default=None, record=None, **kw):
			
		self._option_defaults(record)
		self.type = type
		self.var_name = var_name
		self.label = label
		self.var_type = var_type
		self.default = default
		self.command = record.command
		self.mem = record.mem	
	
	# _option_defaults:
	#	Set default values necessary for all
	#	options. Call this method from the 
	#	__init__ method as shown above, before
	#	the option specific variables so that 
	#	they are not overwritten by this method.
	#
	def _option_defaults(self, record=None, **kw):
		self.widgets = {}
		
		self.type = None
		self.var_name = None
		self.label = None
		self.var_type = None
		self.default = None
		
		self.default_mem = None
		self.callback = None
		self.call_args = None
		self.pass_flag = None
		self.static_vars = ''
		self.variable = None
		
		self.state = NORMAL
		self.color_default = None
		
		self.record = record
		self.command = record.command
		self.mem = record.mem
		
		# list to keep all the event bindings for this option
		# 	in the shape [(event, callback, args),(next)]
		self.binding_list = []
		
		# make a copy otherwise this will effect parental options
		# since we're working with a dictionary
		self.tk_config = copy.copy(record.command.tk_config)
		
		# This can be set to TRUE by set_tk_config or
		#	add_tk_config to indicate that this
		#	option has its own tk_config definition.
		self.config_flag = FALSE
		
	#######################################
	#
	# tkCoLa Access Methods
	# 	These can be called through keywords directly
	#	from the command language.
	#
	##############
	
	#
	#	keyword: 	disabled
	#			set_state |state
	#		
	def set_state(self, state=DISABLED, **kw):
		if state in TRUE_LIST:
			self.state = NORMAL
		elif state in TK_ENABLED:
			self.state = NORMAL
		else:
			self.state = DISABLED
			
	#
	#	keyword:	color_default |where
	#				where is foreground
	#				background- of widget
	#
	def set_color_default(self, where='background', **kw):
		if where in TK_FOREGROUND:
			where = 'foreground'
			
		self.color_default = where
		
	#
	#	keyword:	bind |event |callback
	#				bind event to callback 
	#				for widgets in option
	#
	def set_binding(self, event=None, callback=None, record=None, **kw):
		call = prefix_process(callback, record.command_prefix)
		callback, args = self.set_callback(call)
		
		event = parse_eventkey(event)
		if event:
			self.binding_list.append((event, callback, args))
	
	#
	#	keyword:	callback
	#		
	def make_callback(self, callback=None, pass_flag=TRUE, record=None, **kw):
		call = prefix_process(callback, record.command_prefix)
		self.callback, self.call_args = self.set_callback(call)
		
		if (pass_flag in TRUE_LIST) or (pass_flag == 1):
			self.pass_flag = TRUE
		elif (pass_flag in FALSE_LIST) or not pass_flag:
			self.pass_flag = FALSE
		else:
			self.pass_flag = TRUE
	
	#
	#	keyword:	add_tk_config
	#			set_tk_config
	#
	#	Sets the specified tk_config for the option, or adds
	#		it to the one already in place.
	#		
	def add_tk_config(self, tk_config=None, record=None, **kw):
		self.tk_config = parse_config(tk_config, self.tk_config, self.command.manager)
		self.config_flag = TRUE
		
	def set_tk_config(self, tk_config=None, record=None, **kw):
		self.tk_config = parse_config(tk_config, {}, self.command.manager)
		self.config_flag = TRUE
			
	#######################################
	#
	# User Access Methods
	# 	These methods may be useful called from Python
	#	or tkCoLa. See documentation for examples of use.
	#
	##############
		
	def toggle(self, **kw):
		if self.state == NORMAL:
			self.put_state(state=DISABLED)
		else:
			self.put_state(state=NORMAL)
				
	def put_state(self, state=DISABLED, **kw):
		index = 0
		for key in self.widgets.keys():
			if state == NORMAL:
				if self.real_config[index]:
				    try:
					self.widgets[key].config(foreground=self.real_config[index])
				    except TclError:
					pass
				try:
					self.widgets[key].config(state=NORMAL)
				except TclError:
					pass
			else:
				try:
					self.widgets[key].config(foreground=TK_INACTIVE)
				except TclError:
					pass
				try:
				    	self.widgets[key].config(state=DISABLED)
				except TclError:
				    	pass
						
			self.widgets[key].update()
			index = index + 1
			
		if state == DISABLED:
			self.state = DISABLED
		else:
			self.state = NORMAL
			
	def set_config(self, config_dict={}, **kw):
		for key in self.widgets.keys():
			for con in config_dict.keys():
				if self.widgets[key].config().has_key(con):
					try:
						self.widgets[key].config({con:config_dict[con]})
					except TclError:
						pass
						# raise CommandRunError(0, self.command, self, 'set_config')
			self.widgets[key].update()
	
	# allows configuration of the option through
	# aliasing the kw dictionary
	# for example if the alias_dict looks like this:
	#
	#	alias_dict={'thecolor':'background'}
	#
	#	and there's a corresponding kw['thecolor']
	#	the option will be configured as:
	#
	#	background=kw['thecolor']
	#				
	def alias_config(self, alias_dict={}, **kw):
		config_dict = {}
		for key in alias_dict.keys():
			if kw.has_key(key):
				config_dict[alias_dict[key]] = kw[key]
		if config_dict:
			self.set_config(config_dict=config_dict)
	
	def save_config(self, **kw):
		self.real_config = []
		for key in self.widgets.keys():
			foreground = None
			try:
				foreground = self.widgets[key].cget('foreground')
			except TclError:
				pass
			self.real_config.append(foreground)
			
	#######################################
	#
	# Option Core Methods
	# 	These methods are mainly for internal use.
	#
	##############	
				
	def set_var(self, list_var):
		split = string.split(list_var, '$')
		ret1 = split[0]
		try:
			ret2 = split[1]
		except IndexError:
			ret2 = None
		return ret1, ret2
		
	def set_callback(self, callback):
		if callback:
		    call = callback[0]
		    call_args = callback[1]
		    return call, call_args
		else:
		    return None, None
		
	def handle_callback(self, callback=None, call_args=None, pass_flag=None):
		callback = callback or self.callback
		call_args = call_args or self.call_args
		pass_flag = pass_flag or self.pass_flag
		self.command.handle_callback(self, callback=callback, 
				call_args=call_args, pass_flag=pass_flag)
	
	def handle_call(self, callback=None, call_args=None, pass_flag=None):
		callback = callback or self.callback
		call_args = call_args or self.call_args
		pass_flag = pass_flag or self.pass_flag
		self.command.handle_callback(self, callback=callback, 
				call_args=call_args, pass_flag=pass_flag)
			
	def configure_widgets(self, config_dict=None):
		if not config_dict:
			config_dict = self.tk_config
		for key in self.widgets.keys():
			config_match(self.widgets[key], config_dict, configure=1)
			
	def configure_bindings(self, **kw):
		if not self.binding_list:
			return
		for (event, call, args) in self.binding_list:
			for key in self.widgets.keys():
				self.widgets[key].bind(event, 
					    lambda event=None, self=self, call=call, args=args:
					    self.handle_call(callback=call, call_args=args, pass_flag=FALSE))
				    
	def draw_option(self, frame, config_dict=None, frame_opts={}):
		if not config_dict:
			config_dict = self.tk_config
		
		return_value = self.draw_widgets(frame, config_dict, frame_opts)	
		self.update_default()
			
		if self.callback:
			config_dict['command'] = self.handle_callback
					
		self.configure_widgets(config_dict)
		self.configure_bindings()	
		
		if self.color_default:
			self.set_config(config_dict={self.color_default:eval(self.get()[1])})
				
		self.save_config()
		
		if self.state == DISABLED:
			self.put_state(state=DISABLED)
		
		return return_value
		
	def draw_widgets(self, frame=None, config_dict=None, frame_opts=None):
		# method to draw the widgets for this option
		pass
	
	def update_default(self):
		# method to update default values and display for this option
		default = parse_default(self.default, self.var_type, self.command)
		if not default:
			print 'Command Error, option: variable %s not found' % self.default[1:]
		elif default == 1:
			print 'Command Error, option: variable %s is wrong type' % self.default[1:]
			default = None
		self.variable.set(default)
		
	def get(self):
		# return method for this option
		value = self.variable.get()
		
		# if this option is turned on remember the gotten
		#	value as the default for next time.
		if self.mem:
			self.default_mem = value
				
		# value needs to be a string in any case (for eval function)
		# but if its a string variable it needs to be enclosed in quotes
		if self.var_type == 'string':
			value = ('\"%s\"' % value)	
		
		return self.var_name, value
		
class NoVar_Option(Option):
	def update_default(self):
		pass
	def get(self):
		pass
	
class Button_Option(Option):
	def __init__(self, type='button', var_name=None, label=None, callback=None, 
			pass_flag=None, var_type=None, default=None, 
			record=None, color_default=None, **kw):
			
		self._option_defaults(record)	
		
		self.type = type
		self.var_name = var_name
		self.var_type = get_type(var_type)
		if not self.var_type and var_type:
			raise CommandParseError(self.type, 'var_type', 
						('Unrecognized type %s' % var_type))
			return None
			
		self.default = default
		self.label = label
		# set the callback for the button
		call = prefix_process(callback, record.command_prefix)
		self.callback, self.call_args = self.set_callback(call)
		
		if (pass_flag in TRUE_LIST) or (pass_flag == 1):
			self.pass_flag = 1
		else:
			self.pass_flag = 0
		
		self.variable = default
		self.color_default = color_default
		
	def handle_callback(self):
		self.variable = self.command.handle_callback(option=self, pass_flag=self.pass_flag)
				
	def draw_widgets(self, frame, config_dict=None, frame_opts={}):
		self.variable = self.default
		self.widgets['_button'] = Button(frame, text=self.label)
		self.widgets['_button'].pack(frame_opts)
	
	def update_default(self):
		pass
		
	def get(self):
		if not self.var_name:
			return
		
		# Note: to use for a passback from a command window
		# which can be tricky, pass the option as the
		# data to open the window then modify the option.variable
		# with the command_return_object from the other
		# window- this will only work for one variable though
		
		# button returns may be in the form
		# of the variable or a dictionary
		# containing many values as with the
		# command_manager manager_ok method
		if type(self.variable) == DictType:
		    value = None
		    for key in self.variable.keys():
			if key == self.var_name:
				value = self.variable[key]
		else:	 
			value = self.variable
			
		if not value:
			value = self.default
			
		# if this option is turned on remember the gotten
		#	value as the default for next time.
		if self.mem:
			self.default_mem = value
				
		# value needs to be a string in any case (for eval function)
		# but if its a string variable it needs to be enclosed in quotes
		if self.var_type == 'string':
			value = ('\"%s\"' % value)
			
		return self.var_name, value
			
class Entry_Option(Option):
	def __init__(self, type='entry', var_name=None, label=None, 
			var_type=None, default=None, record=None, **kw):
			
		self._option_defaults(record)	
		
		self.type = type
		self.var_name = var_name
		self.label = label
		self.var_type = get_type(var_type)
		if not self.var_type:
			raise CommandParseError(self.type, 'var_type', 
						('Unrecognized type %s' % var_type))
			return None
			
		self.default = default

	def draw_widgets(self, frame, config_dict=None, frame_opts={}):
		self.widgets['_entry_frame'] = Frame(frame)
		self.widgets['_entry_frame'].pack(frame_opts, fill=X, expand=1, anchor=W)
		
		if not self.default == None:
			e_width = len(str(self.default)) + 4
			if e_width > 50:
				e_width = 50
		else:
			e_width = 10
		
		self.widgets['_label'] = Label(self.widgets['_entry_frame'],
							 text=self.label)
		self.widgets['_label'].pack(side='left')
		self.widgets['_entry'] = self.variable = Entry(self.widgets['_entry_frame'], 
							width=e_width,
							foreground='black',
							background='grey91')
		self.variable.pack(side='left')

	def update_text(self, insert_text='', **kw):
		self.variable.delete(0,END)
		self.variable.insert(END, insert_text)
		
	def update_default(self):
		if self.default_mem:
			default = self.default_mem
		else:
			default = parse_default(self.default, self.var_type, self.command)
			
			if not default:
				print 'Command Error, option: variable %s not found' % self.default[1:]
				default = ''
			if default == 1:
				print 'Command Error, option: variable %s is wrong type' % self.default[1:]
				default = ''
		self.variable.delete(0,END)
		self.variable.insert(END, default)	
	
	def get(self):
		try:
		    if self.var_type == 'int':
			    v = string.atoi(self.variable.get())
		    elif self.var_type == 'float':
			    v = string.atof(self.variable.get())
		except ValueError:
		    print 'Please enter a %s in the %s box' % (self.var_type, self.label)
		    return VAL_ERROR
		
		value = self.variable.get()
		
		# if this option is turned on remember the gotten
		#	value as the default for next time.
		if self.mem:
			self.default_mem = value
				
		# value needs to be a string in any case (for eval function)
		# but if its a string variable it needs to be enclosed in quotes
		if self.var_type == 'string':
			value = ('\"%s\"' % value)
				
		return self.var_name, value
			 		
class Check_Option(Option):
	def __init__(self, type='check', var_name=None, label="", 
			var_type='i', on_value='1', off_value='0',
			default='0', record=None, **kw):
				
		self._option_defaults(record)	
		
		self.type = type
		self.var_name = var_name
		self.label = label
		self.var_type = get_type(var_type)
		if not self.var_type:
			raise CommandParseError(self.type, 'var_type', 
						('Unrecognized type %s' % var_type))
			return None
			
		self.on_value = on_value
		self.off_value = off_value
		if default in TRUE_LIST or (default == on_value):
			self.default = 'on'
		elif default in FALSE_LIST or (default == off_value):
			self.default = 'off'
		
	def draw_widgets(self, frame, config_dict=None, frame_opts={}):
		if self.var_type in ('string', 'float'):
			self.variable = StringVar(frame)
		elif self.var_type == 'int':
			self.variable = IntVar(frame)
		
		self.widgets['_check'] = Checkbutton(frame,
						    variable=self.variable,
						    text=self.label,
						    onvalue=self.on_value,
						    offvalue=self.off_value)
		self.widgets['_check'].pack(frame_opts, anchor=W)
		
	def update_default(self):
		if self.default_mem:
			default = self.default_mem
		else:
			default = parse_default(self.default, self.var_type, self.command)
			if not default:
				print 'Command Error, option: variable %s not found' % self.default[1:]
				default = 'off'
			elif default == 1:
				print 'Command Error, option: variable %s is wrong type' % self.default[1:]
				default = 'off'
					
		if default in TRUE_LIST:
			if self.callback:
				self.widgets['_check'].invoke()
				self.handle_callback()
			else:
				self.widgets['_check'].select()
		else:
			if self.callback:
				self.widgets['_check'].select()
				self.widgets['_check'].invoke()
				self.handle_callback()
			else:
				self.widgets['_check'].deselect()
		
	def get(self):
		value = str(self.variable.get())
		
		# if this option is turned on remember the gotten
		#	value as the default for next time.
		if self.mem:
			if value == self.on_value:
				self.default_mem = 'on'
			else:
				self.default_mem = 'off'
				
		# value needs to be a string in any case (for eval function)
		# but if its a string variable it needs to be enclosed in quotes
		if self.var_type == 'string':
			value = ('\"%s\"' % value)
				
		return self.var_name, value
	
	# TkCoLA Called Methods	
	def add_link(self, on_list=None, off_list=None, 
			record=None, **kw):	
		
		flag = FALSE
		if on_list:	
			on_list = map(string.strip, string.split(on_list, ","))
			flag = TRUE
		if off_list:		
			off_list = map(string.strip, string.split(off_list, ","))
			flag = TRUE
		if not flag:
			return
		
		keys = "command.link_vars$"
		
		if on_list:
			keys = "%son_vars=(" % keys
			for link in on_list:
				keys = "%s\'%s\'," % (keys, link)
			keys = "%s)" % keys
		
		if off_list:
			if on_list:
				keys = "%s," % keys
				
			keys = "%soff_vars=(" % keys
			for link in off_list:
				keys = "%s\'%s\'," % (keys, link)
			keys = "%s)" % keys
			
		self.make_callback(callback=keys, pass_flag=FALSE, record=record)
		    		
class Radio_Option(Option):
	def __init__(self, type='radio', var_name=None, label=None, 
			var_type=None, default=None, orientation=None,
			record=None, **kw):
			
		self._option_defaults(record)
		self.radio_options_list = []
		
		self.type = type
		self.var_name = var_name
		self.label = label
		self.var_type = get_type(var_type)
		if not self.var_type:
			raise CommandParseError(self.type, 'var_type', 
						('Unrecognized type %s' % var_type))
			return None
			
		self.default = default
		if orientation in TK_HORIZ:
			self.orientation = FALSE
		else:
			self.orientation = TRUE
		
	def option_add(self, label=None, value=None, **kw):
		try:
		    if self.var_type == 'int':
		    	value = string.atoi(value)
		    elif self.var_type == 'float':
		    	value = string.atof(value)
		except ValueError:
		    print 'Inappropriate type for radopt in radiogroup <%s>' % self.label
		    # Give this a try
		    self.type = 'string'
		    
		rad_tup = (label, value)
		self.radio_options_list.append(rad_tup)
		
	def draw_widgets(self, frame, config_dict=None, frame_opts={}):
		# Don't display if the number of options is
		# 0 or 1, not good numbers for radio buttons
		if len(self.radio_options_list) < 2:
			return
			
		self.widgets['_radio_frame'] = frame = Frame(frame, relief=GROOVE, bd=2)
		self.widgets['_radio_frame'].pack(frame_opts, anchor=W)
		
		if self.label:
		    self.widgets['_label'] = Label(frame, text=self.label)
		    self.widgets['_label'].pack(side=TOP, pady=5)
		
		if self.var_type == 'string':
			self.variable = StringVar(frame)
		elif self.var_type == 'int':
			self.variable = IntVar(frame)
		else:
			self.variable = Variable(frame)
		
		if self.orientation:
			side = TOP
		else:
			side = LEFT
		
		for option in self.radio_options_list:
			label = option[0]
			value = option[1]
			self.widgets[label+'_button'] = Radiobutton(frame,
							 variable=self.variable,
							 text=label,
							 value=value)
			self.widgets[label+'_button'].pack(side=side, anchor=W)
			
	def update_default(self):
		if self.default_mem:
			default = self.default_mem
		else:
			# needs to be updated for variable handling
			default = self.default
		self.widgets[default+'_button'].invoke()
		
	def get(self):
		# return method for this option
		value = self.variable.get()
		
		# if this option is turned on remember the gotten
		#	value as the default for next time.
		if self.mem:
			try:
			    for option in self.radio_options_list:
				    if str(value) == option[1]:
					self.default_mem = option[0]
			except KeyError:
			    pass
				
		# value needs to be a string in any case (for eval function)
		# but if its a string variable it needs to be enclosed in quotes
		if self.var_type == 'string':
			value = ('\"%s\"' % value)	
		
		return self.var_name, value
		
class Scale_Option(Option):
	def __init__(self, type='scale', var_name=None, label=None, 
			var_type=None, from_val=None, to_val=None,
			increment=None, default=None, orientation=None,
			record=None, **kw):
			
		self._option_defaults(record)	
		
		self.type = type
		self.var_name = var_name
		self.label = label
		self.var_type = get_type(var_type)
		if not self.var_type:
			raise CommandParseError(self.type, 'var_type', 
						('Unrecognized type %s' % var_type))
			return None
		
		if self.var_type == 'string':
			raise CommandParseError(self.type, 'var_type', 'Type must be int or float')
			return None
		
		try:
			self.from_val = eval(from_val)
			self.to_val = eval(to_val)
			self.increment = eval(increment)
		except NameError:		# FIXME: this opens a hole for errors
			raise CommandParseError(self.type, 'parameters', 'Type must be int or float')
			return None
			
		self.default = default
		if orientation in TK_VERT:
			self.orientation = VERTICAL
		else:
			self.orientation = HORIZONTAL
		
	def draw_widgets(self, frame, config_dict, frame_opts={}):
		self.variable = self.widgets['_scale'] = Scale(frame, 
					  label=self.label,
					  from_ = self.from_val, 
					  to = self.to_val,
					  resolution = self.increment,
					  orient = self.orientation)
		self.variable.pack(frame_opts, anchor=W)
	
	def update_default(self):
		if self.default_mem:
			default = self.default_mem
		else:
			default = parse_default(self.default, self.var_type, self.command)
			if not default:
				print 'Command Error, option: variable %s not found' % self.default[1:]
				default = ''
			if default == 1:
				print 'Command Error, option: variable %s is wrong type' % self.default[1:]
				default = ''
		try:
		    if self.var_type == 'float':
			    try:
				self.digits = len(string.split(default, '.')[1])
			    except IndexError:
			    	self.digits = 0
			
			    default = string.atof(default)
		    if self.var_type == 'int':
		    	    self.digits = 0
			    default = string.atoi(default)
		    else:
			    self.digits = 0
			    default = None
			    
		except ValueError:
		     print 'Error in scale definition'
		     return None
		
		self.widgets['_scale']['digits'] = self.digits      
		self.variable.set(default)
		
	def get(self):
		# return method for this option
		value = self.variable.get()
		
		# if this option is turned on remember the gotten
		#	value as the default for next time.
		if self.mem:
			self.default_mem = str(value)
				
		# value needs to be a string in any case (for eval function)
		# but if its a string variable it needs to be enclosed in quotes
		if self.var_type == 'string':
			value = ('\"%s\"' % value)	
		
		return self.var_name, value    
		
class List_Option(Option):
	def __init__(self, type='list', var_name=None, label=None, var_type=None,
			default=None, list=None, select_action=None,
			record=None, **kw):
				
		self._option_defaults(record)	
		
		self.type = type
		self.var_name = var_name
		self.label = label
		if var_type:
			self.var_type = get_type(var_type)
			if not self.var_type:
				raise CommandParseError(self.type, 'var_type', 
							('Unrecognized type %s' % var_type))
				return None
		else:
			self.var_type = None
			
		if self.type[0:5] == 'multi':
			self.mode = EXTENDED
		else:
			self.mode = BROWSE
			
		self.default = default
		self.list_var, self.list_attr = self.set_var(list)
		
		call = prefix_process(select_action, record.command_prefix)
		self.callback, self.call_args = self.set_callback(call)
		
	def draw_widgets(self, frame, config_dict, frame_opts={}):
		self.widgets['_frame'] = Frame(frame)
		self.widgets['_frame'].pack(frame_opts, anchor=W)
		self.widgets['y_scroll'] = Scrollbar(self.widgets['_frame'], width=10)
		self.widgets['y_scroll'].pack(side=LEFT, fill=Y, expand=1)
		self.widgets['_box'] = Listbox(self.widgets['_frame'], selectmode=self.mode,
						exportselection=0, setgrid=1)
		self.widgets['_box'].pack(side=RIGHT, fill=BOTH, expand=1)
		self.widgets['_box']['yscrollcommand'] = self.widgets['y_scroll'].set
		self.widgets['y_scroll']['command'] = self.widgets['_box'].yview
		
		self.widgets['_box'].bind('<ButtonRelease-1>', self.select_action)
		self.widgets['_box'].bind('<KeyRelease-Up>', self.select_action)
		self.widgets['_box'].bind('<KeyRelease-Down>', self.select_action)
		
		self.fill_box()
		self.widgets['_box'].see(0)
		self.widgets['_box'].select_set(0)
		self.select_action()
		
	def fill_box(self):
		try:
			# evaluate at the level of the command
			self.list = list = self.command.eval_var_string(self.list_var)
		except (NameError, AttributeError):
			print 'Command Error, list_option: List Variable %s does not exist.' % self.list_var
			return
			
		# get a list from the variable
		if type(list) in (ListType, TupleType):
		    self.list_type = 'list'
		    for member in list:
			if self.list_attr:
			    try:
				value = member.__dict__[self.list_attr]	
			    except KeyError:
				print ('Command Error, entry_gen: %s does not have attribute %s' %
					(self.list_var, self.list_attr))
				return
			else:
			    value = member
			
			self.widgets['_box'].insert(END, str(value))
			
		elif type(list) == DictType:
		    self.list_type = 'dict'
		    for key in list.keys():
		    	member = list[key]
			if self.list_attr:
			    try:
			    	value = member.__dict__[self.list_attr]
			    except KeyError:
			    	print ('Command Error, entry_gen: %s does not have attribute %s' %
					(self.list_var, self.list_attr))
				return
			else:
			    value = member
			    
			self.widgets['_box'].insert(END, str(value))
		else:
		    print 'Command Error, list_option: %s is not a list, tuple or dict type' % self.list_var
		    return	
			
	def select_action(self, *args, **kw):
		select = self.widgets['_box'].curselection()
		if len(select) == 0:
			return
		
		# m is for multi
		if (self.type[0] == 'm') and (len(select) > 1):
			index = '('
			label = '('
			for i in range(0, len(select)):
				n = string.atoi(select[i])
				index = '%s%d,' % (index, n)
				label = '%s\'%s\',' % (label, self.widgets['_box'].get(n))
			# strip trailing comma
			index = index[0:(len(index)-1)] + ')'
			label = label[0:(len(label)-1)] + ')'
				
		else:
			n = string.atoi(select[0])
			index = select[0]
			label = '\'%s\'' % self.widgets['_box'].get(n)
		
		# if we're here this should be true but...
		if self.callback:
			# we'll send these values along as tagalongs :)
			if self.call_args:
				call_args = '%s, ' % self.call_args
			else:
				call_args = ''
			
			call_args =('%s index=%s, label=%s' % 
						(call_args, index, label))
			self.handle_callback(call_args=call_args)
			
	def update_default(self):
		pass
	
	def get(self, **kw):
		pass
			    		
class Menu_Option(Option):
	def __init__(self, type='menu', var_name=None, label=None, var_type=None, 
			fill_callback=None, tearoff=None, display_keys=TRUE, default=None,
			callback=None, record=None, **kw):
				
		self._option_defaults(record)
		self.pass_flag = TRUE	
			
		self.item_list = []
		self.original_items = []
		
		# Width (in chars) of the longest item label
		self.max_width = 0
		self.display_keys = display_keys
		
		self.type = type
		self.var_name = var_name
		self.label = label
		call = prefix_process(callback, record.command_prefix)
		self.menu_call, self.menu_args = self.set_callback(call)
		call = prefix_process(fill_callback, record.command_prefix)
		self.fill_callback, self.fill_args = self.set_callback(call)
		
		if tearoff in TRUE_LIST:
			self.tearoff = TRUE
		else:
			self.tearoff = FALSE
				
		self.default = default
		
		if var_type:
			self.var_type = get_type(var_type)
			if not self.var_type:
				raise CommandParseError(self.type, 'var_type', 
							('Unrecognized type %s' % var_type))
				return None
		else:
			self.var_type = None
		
	def add_menu_item(self, label=None, value=None, command=None, 
				key=None, record=None, prefix=None, **kw):
		# if we have a menu callback then
		# ignore the command for this item
		prefix = prefix or record.command_prefix
		if command and not self.menu_call:	
			callback = prefix_process(command, prefix)
			callback, args = self.set_callback(callback)
		else:
			callback = None
			args = None
		
		item_list = self.item_list	
		if self.fill_callback:
			item_list = self.original_items
		
		if len(label) > self.max_width:
			self.max_width = len(label)
					
		item_list.append((label, value, callback, args, key))
		
	def add_menu_separator(self, **kw):
		item_list = self.item_list	
		if self.fill_callback:
			item_list = self.original_items
		item_list.append(('<<Separator>>', 0, None, None, None))
		
	def handle_menu(self, index):
		index = index - 1
		label, value, callback, args, key = self.item_list[index]
		if self.var_name:
			self.variable = value
		if self.type == 'pop':
			self.widgets['_menu_button']['text'] = label
			self.widgets['_menu_button'].update()
			
		if callback:
			self.callback = callback
			if args:
				comma = ','
			else:
				comma = ''
				
			# we'll send these values along as tagalongs :)
			self.call_args = args + ('%s menu_index=%d, menu_label=\'%s\''
						% (comma, index, label))
			self.handle_callback()
			
		elif self.menu_call:
			self.callback = self.menu_call
			self.call_args = self.menu_args
			self.handle_callback()
		
	def draw_widgets(self, frame, config_dict, frame_opts={}):
		label = self.label
		
		self.widgets['_menu_button'] = Menubutton(frame, text=label)
		self.widgets['_menu_button'].pack(frame_opts)
		self.widgets['_menu'] = m = Menu(self.widgets['_menu_button'], 
						tearoff=self.tearoff)
		if self.fill_callback:
			self.handle_menu_fill()
		else:
			self.menu_update()
			
		if type(self.default) in (StringType, FloatType, IntType):
		    for item in self.item_list:
			    if self.default == item[1]:
				    self.widgets['_menu_button']['text'] = item[0]
				    self.variable = item[1]
				    break
				
		self.widgets['_menu_button']['menu'] = self.widgets['_menu']
		if self.var_name:
			self.update_default()
			
	def update_menu(self, **kw):
		menu = self.widgets['_menu']
		menu.delete(0, END)
		if self.fill_callback:
			self.handle_menu_fill()
		else:
			self.menu_update()
		menu.update()	
							
	def handle_menu_fill(self):
		# making things easier from inside tkCoLa
		parent = self.command.parent
		manager = self.command.manager
		command = self.command
		option = self
		
		fill_args = self.fill_args or ''
		eval(self.fill_callback + '(' + self.fill_args + ',' + 'menu=self.widgets[\'_menu\'])')
	
		# reset existing menu items and add them to the menu
		if self.original_items:
			self.menu_fillitems(item_list=self.original_items, index=len(self.item_list))
			self.item_list = self.item_list + self.original_items
		
	def menu_fillitems(self, item_list=None, index=None, **kw):
		item_list = item_list or self.item_list
		if not item_list:
			return
		menu = self.widgets['_menu']
		index = index or 0
		for item in item_list:
			label = item[0]
			key = item[4]	
			index = index + 1
			accel = ""
			if key and self.display_keys:
				accel = key
				# Note: Adds binding to the toplevel
				top = self.widgets['_menu_button'].winfo_toplevel()
				event = parse_eventkey(key)
				if event:
				    top.bind(event, lambda event=None, self=self,
						index=index: self.handle_menu(index))
			
			if label == '<<Separator>>':
				menu.add_separator()
				continue
			menu.add_command(label=label, accelerator=accel, 
			command =
			lambda event=None, self=self, index=index:
				self.handle_menu(index))		
		menu.update()
		
	def menu_update(self, **kw):
		menu = self.widgets['_menu']
		menu.delete(0, END)
		self.menu_fillitems()	
		
	def update_default(self):
		for item in self.item_list:
			# needs to be updated for variable handling
			if item[0] == self.default:
				self.variable = item[1]
	
	def get(self):
		if not self.var_name:
			return
			
		# return method for this option
		value = self.variable
		
		# if this option is turned on remember the gotten
		#	value as the default for next time.
		if self.mem:
			try:
			    for item in self.item_list:
			    	    label = item[0]
				    if str(value) == item[1]:
					self.default_mem = item[0]
			except KeyError:
			    pass
				
		# value needs to be a string in any case (for eval function)
		# but if its a string variable it needs to be enclosed in quotes
		if self.var_type == 'string':
			value = ('\"%s\"' % value)	
		
		return self.var_name, value
		
	# Built-in Callback for the fill_callback to fill the menu with a list
	def menu_fill_list(self, list=[], label_var=None, value_var=None, **kw):
		self.item_list = []
		if type(list) in (ListType, TupleType):
		    if not list:
			self.item_list.append(("No Items", "", None, None, None))
			self.menu_update()
			return
		    straight = TRUE
		    if type(list[0]) == TupleType:
			    straight = FALSE
		    index = 0
		    for item in list:
			if not straight:
			    item = item[0]
			    value = item[1]	
			if (type(item) == InstanceType) and (type(label_var) == StringType):
			    label = item.__dict__[label_var]
			else:
			    label = item
			if (type(item) == InstanceType) and (type(value_var) == StringType):
			    value = item.__dict__[value_var]
			    
			# value is index
			elif (type(item) == InstanceType) and value_var == 1:
				value = "%d" % index
			else:
			    if straight:
				value = item
			
			index = index + 1	
			self.item_list.append((label, value, None, None, None))
			
		    self.menu_update()
	
	# Built-in Callback to make a file_list menu
	def menu_fill_files(self,pat="*", path=".", **kw):
		self.item_list = []
		pat = pat or "*"
		path = path or "."
		files = os.listdir(path)
		for file in files:
			if fnmatch.fnmatch(file, pat):
				self.item_list.append((file,file,None,None,None))
		self.menu_update()

class Text_Option(Option):
	def __init__(self, type='text', label=None, default=None, 
			var_name=None, record=None, **kw):
		
		self._option_defaults(record)	

		self.type = type
		# find a variable string (ie. $var_name$ ) if present
		index = string.find(label, '$')		# start
		if index > 0:
			self.var_name = string.split(label[index:], '$')[1]
		else:
			self.var_name = var_name
			
		self.label = label
		self.default = default
		
	def draw_widgets(self, frame, config_dict, frame_opts={}):
		self.text_variable = StringVar(frame)
		self.widgets['_text'] = Label(frame, text=self.label, 
					textvariable=self.text_variable)
		self.widgets['_text'].pack(frame_opts, anchor=W)
		self.text_variable.set(self.label)
	
	def update_default(self):
		if self.var_name:
			if not self.default:
				default = ''
			else:
				default = self.default
			index = string.find(self.label, '$' + self.var_name + '$')
			length = len('$' + self.var_name + '$')
			# modify the label with the original text
			start = self.label[0:index]
			end = self.label[(index+length):]
			self.text_variable.set(start + default + end)
		
	def get(self):
		pass
									
class Frame_Option(Option):
	def __init__(self, type='frame', label=None, fill_callback=None,
			var_name=None, var_type=None,
			get_value_callback=None, record=None, **kw):
			
		self._option_defaults(record)	
		
		self.type = type
		self.label = label

		call = prefix_process(fill_callback, record.command_prefix)
		self.callback, self.call_args = self.set_callback(call)
		self.pass_flag = TRUE
		
		self.var_name = var_name
		if var_type:
			self.var_type = get_type(var_type)
			if not self.var_type:
				raise CommandParseError(self.type, 'var_type', 
							('Unrecognized type %s' % var_type))
				return None
		else:
			self.var_type = None
			
		call = prefix_process(get_value_callback, record.command_prefix)
		self.get_call, self.get_args = self.set_callback(call)
		
	def draw_widgets(self, frame, config_dict, frame_opts={}):
		self.variable = self.default
		self.widgets['_frame'] = Frame(frame)
		self.widgets['_frame'].pack(frame_opts)
		
		args = self.call_args
		if self.call_args:
			self.call_args = self.call_args + ','
			
		# pass this frame to the callback which
		# should handle the filling of the frame
		self.call_args = self.call_args + 'frame=option.widgets[\'_frame\']'
		self.handle_callback()
		self.call_args = args
		
	def configure_widgets(self, config_dict=None):
		if not config_dict:
			config_dict = self.tk_config
		for key in self.widgets.keys():
			config_match(self.widgets[key], config_dict, configure=1)
		
		# for this option also configure the children
		for w in self.widgets['_frame'].winfo_children():
			config_match(w, config_dict, configure=1)
		
	def update_default(self):
		pass
		
	def handle_callback(self):
		# creating something in the frame may return
		# a useful object for our users such as
		# creating a Class Instance such as Color_Picker
		self.object =  self.command.object = self.command.handle_callback(self)
		
		# FIXME: this needs to be the case for
		#		objects referenced in the 
		#		command but a list won't work
		# 		since everytime this command
		#		is invoked the list will get
		#		longer!
		# add this object to the commands list
		# self.command.object.append(object)
		
	def get(self):
		if not self.var_name:
			return
				
		try:	
			value = self.command.eval_var_string(self.get_call, option=self)
		except (AttributeError, NameError):
		    if self.variable:
			value = self.variable
		    else:
			print 'Command Error, frame option: Variable %s not found' % self.var_name
			return
				
		# value needs to be a string in any case (for eval function)
		# but if its a string variable it needs to be enclosed in quotes
		if self.var_type == 'string':
			value = ('\"%s\"' % value)
				
		return self.var_name, value
						
class Format_Option(NoVar_Option):
	def __init__(self, type='row', label=None, in_frame=None, padx='5', pady='5', 
			side=None, record=None, **kw):
			
		self._option_defaults(record)	
		
		self.type = type
		self.label = label
		self.in_frame = in_frame
		
		if padx:
			try:
			    self.padx = string.atoi(padx)
			except ValueError:
			    raise CommandParseError(type, 'padx', 'default value used')
			    self.padx = 5
		else:
			self.padx = 5
			
		if pady:
			try:
			    self.pady = string.atoi(pady)
			except ValueError:
			    raise CommandParseError(type, 'pady', 'default value used')
			    self.pady = 5
		else:
			self.pady = 5
					
		self.side = None
		if side:
		    for key in TK_SIDES.keys():
			    if side in TK_SIDES[key]:
				    self.side = key
				    break
		    if not self.side:
			    raise CommandParseError(type, 'side', 
						    ('unknown value %s: default used' % side))
		if not self.side:
			if type == 'row':
				self.side = TOP
			else:
				self.side = LEFT
		
	def draw_widgets(self, frame, config_dict=None, frame_opts={}):
		self.widgets['_frame'] = Frame(frame)
		self.configure_widgets(config_dict)
		return self.widgets['_frame']
	
class Label_Option(NoVar_Option):
	def __init__(self, type='label', label=None, record=None, **kw):
			
		self._option_defaults(record)	
		
		self.type = type
		self.label = label
		
	def draw_widgets(self, frame, config_dict=None, frame_opts={}):
		self.widgets['_frame'] = Frame(frame, relief=GROOVE, borderwidth=2)
		self.widgets['_frame'].pack(side=BOTTOM, fill=X, expand=1, pady=5)
		
		# this needs to be adjusted to reflect the actual width of the enlclosing box!
		self.widgets['_label'] = Label(self.widgets['_frame'], text=self.label,
				wraplength=250, justify=LEFT)
		self.widgets['_label'].pack(side=TOP, padx=10, pady=5)


##########################################################################################	
# Generator Options:
# 	Generator options are specified by using a variable type
# 		in the form 'list_' + var_type (eg. list_int)
#		and placing the list variable in the default
#		field. Attributes of list members may be specified
#		using the '$' designator following the list variable
#		if the list is a list (or dictionary) of Instances
#
    		
class Generator_Option(Option):
	def __init__(self, type='generator_type', var_name=None, label=None, var_type=None,
			default=None,record=None, **kw):
			
		self._option_defaults(record)	
			
		self.type = type
		self.var_name = var_name
		self.label = label
		self.list_var, self.list_attr = self.set_var(default)
		self.var_type = get_type(var_type)
		if not self.var_type:
			raise CommandParseError(self.type, 'var_type', 
						('Unrecognized type %s' % var_type))
			return None
		
		self.option_list = []
				
	def draw_widgets(self, frame, config_dict, frame_opts={}):
		# erase old option list
		del self.option_list
		self.option_list = []
		n = 0
		
		try:
			# evaluate at the level of the command
			list = self.command.eval_var_string(self.list_var)
		except (NameError, AttributeError):
			print 'Command Error, entry_gen: List Variable %s does not exist.' % self.list_var
			return
		
		# make a list of entry options to be managed
		if type(list) in (ListType, TupleType):
		    self.list_type = 'list'
		    for member in list:
		    	if self.label:
				label = '%s %d' % (self.label, n)
			else:
				label = ''
			if self.list_attr:
			    try:
				value = member.__dict__[self.list_attr]
			    except KeyError:
				print ('Command Error, entry_gen: %s does not have attribute %s' %
					(self.list_var, self.list_attr))
				return
			    if self.label:
			    	label = '%s %d' % (self.label, n)
			    else:
				label = ''
		    
			else:
			    value = member
			
			option = self.make_option(var_name='x', label=label, default=str(value))
			
			self.option_list.append(option)
			n = n + 1
			
		elif type(list) == DictType:
		    self.list_type = 'dict'
		    for key in list.keys():
		    	member = list[key]
			if self.label:
				label = '%s %s' % (self.label, key)
			else:
				label = ''
			if self.list_attr:
			    try:
			    	value = member.__dict__[self.list_attr]
			    except KeyError:
			    	print ('Command Error, entry_gen: %s does not have attribute %s' %
					(self.list_var, self.list_attr))
				return
			    if self.label:
			    	label = '%s %s' % (self.label, key)
			    else:
				label = ''
			else:
			    value = member
			
			option = self.make_option(var_name=key, label=label, default=str(value))
			self.option_list.append(option)
			
		else:
		    print 'Command Error, entry_gen: %s is not a list, tuple or dict type' % self.list_var
		    return
		 
		# draw these widgets
		self.command.draw_options(frame, self.option_list, no_label=1, frame_opts=frame_opts)
	
	def make_option(self, var_name=None, label='', default=''):
		# this method makes the individual members'
		# options from the input
		pass
		    
	def update_default(self):
		pass
		
	def get(self):
		if self.list_type == 'list':
		    value_string = '('
		    for opt in self.option_list:
			    varval = opt.get()
			    if not varval:
				continue
			    var, val = varval
			    
			    value_string = '%s%s,' % (value_string, val)
		    value_string = value_string[0:(len(value_string)-1)] + ')'
		else:
		    value_string = '{'
		    for opt in self.option_list:
		    	varval = opt.get()
			if not varval:
				continue
			var, val = varval
			value_string = '%s\'%s\':%s,' % (value_string, var, val)
		    value_string = value_string[0:(len(value_string)-1)] + '}'
		    
		return self.var_name, value_string
		
class Entry_Generator(Generator_Option):
	def __init__(self, type='entry_gen', var_name=None, label=None, var_type=None,
			default=None, record=None, **kw):
			
		self._option_defaults(record)	
			
		self.type = type
		self.var_name = var_name
		self.label = label
		self.list_var, self.list_attr = self.set_var(default)
		self.var_type = get_type(var_type)
		if not self.var_type:
			raise CommandParseError(self.type, 'var_type', 
						('Unrecognized type %s' % var_type))
			return None
		
		self.option_list = []
		
	def make_option(self, var_name=None, label='', default=''):
		entry = Entry_Option(type='entry',var_name=var_name, label=label, var_type=self.var_type, 
					    default=default, record=self.record)
		return entry
		
class Check_Generator(Generator_Option):
	def __init__(self, type='check_gen', var_name=None, label=None, var_type=None,
			default=None, on_value=None, off_value=None,
			record=None, **kw):
			
		self._option_defaults(record)	
			
		self.type = type
		self.var_name = var_name
		self.label = label
		self.list_var, self.list_attr = self.set_var(default)
		self.var_type = var_type
		self.on_value = on_value
		self.off_value = off_value
		
		self.option_list = []
		
	def make_option(self, var_name=None, label='', default=''):
		if default == self.on_value:
			default = 'on'
		else:
			default = 'off'
		check = Check_Option(type='check',var_name=var_name, label=label, 
					var_type=self.var_type, on_value=self.on_value, 
					off_value=self.off_value, default=default,
					record=self.record)
		return check
					
class Button_Generator(Generator_Option):
	def __init__(self, type='button_gen', var_name=None, label=None, 
			callback=None, var_type=None,
			default=None, pass_flag=None,
			record=None, **kw):
			
		self._option_defaults(record)	
			
		self.type = type
		self.var_name = var_name
		self.label = label
		self.list_var, self.list_attr = self.set_var(default)
		self.var_type = var_type
		self.callback = callback
		self.pass_flag = pass_flag
		
		self.option_list = []
		
	def make_option(self, var_name=None, label='', default=''):
		button = Button_Option(type='rebutton',var_name=var_name, var_type=self.var_type,
					default=default, label=label, callback=self.callback,
					pass_flag=self.pass_flag, record=self.record, 
					color_default=self.color_default)
		return button


##########################################################################################	
# File Options:
# 	File Selection Box Option for tkCoLa
#		based on the FileDialog from
#		tkinter. Uses the same key system
#		for storage of default and patterns.
#		Ok command invokes the command managers ok
#		command window method.
#

dialogstates = {}

class Base_File_Option(Option):
	def __init__(self, type='file', var_name=None,
		    multi_file=FALSE, pattern='*', default='',
		    key=None, path_file=TRUE, record=None, **kw):
			
		self._option_defaults(record)	
		
		self.type = type
		self.var_name = var_name
		
		# Flag to return full path with file
		self.path_file = path_file
		
		self.multi_file = multi_file
		if multi_file:
			self.mode = EXTENDED
			self.var_type = 'list'
		else:
			self.mode = BROWSE
			self.var_type = 'string'
			
		self.pattern = pattern
		self.default = default
		self.key = key
		
		self.dir_or_file = os.curdir
			
		self.default = default
		
	def draw_widgets(self, frame, config_dict=None, frame_opts={}):
		main = self.widgets['main'] = Frame(frame)
		main.pack(frame_opts)
		
		b = self.widgets['botframe'] = Frame(main)
		b.pack(side=BOTTOM, fill=X, expand=1)

		self.selection = Entry(main)
		self.selection.pack(side=BOTTOM, fill=X, expand=1)
		self.selection.bind('<Return>', self.ok_event)
	
		self.filter = Entry(main)
		self.filter.pack(side=TOP, fill=X, expand=1)
		self.filter.bind('<Return>', self.filter_command)
	
		mid = self.widgets['midframe'] = Frame(main)
		mid.pack(expand=1, fill=BOTH)
	
		self.filesbar = Scrollbar(mid)
		self.filesbar.pack(side=RIGHT, fill=Y, expand=1)
		self.files = Listbox(mid, exportselection=0,
				     yscrollcommand=(self.filesbar, 'set'),
				     selectmode=self.mode)
		self.files.pack(side=RIGHT, expand=1, fill=BOTH)
		btags = self.files.bindtags()
		self.files.bindtags(btags[1:] + btags[:1])
		self.files.bind('<ButtonRelease-1>', self.files_select_event)
		self.files.bind('<Double-ButtonRelease-1>', self.files_double_event)
		self.filesbar.config(command=(self.files, 'yview'))
	
		self.dirsbar = Scrollbar(mid)
		self.dirsbar.pack(side=LEFT, fill=Y, expand=1)
		self.dirs = Listbox(mid, exportselection=0,
				    yscrollcommand=(self.dirsbar, 'set'))
		self.dirs.pack(side=LEFT, expand=1, fill=BOTH)
		self.dirsbar.config(command=(self.dirs, 'yview'))
		btags = self.dirs.bindtags()
		self.dirs.bindtags(btags[1:] + btags[:1])
		self.dirs.bind('<ButtonRelease-1>', self.dirs_select_event)
		self.dirs.bind('<Double-ButtonRelease-1>', self.dirs_double_event)
	
		# Just Go!
		key = parse_default(self.key,self.var_type,self.command)
		
		if self.key and dialogstates.has_key(key):
			self.directory, file, self.pattern = dialogstates[key]
		else:
			file = ''
			dir_or_file = os.path.expanduser(self.dir_or_file)
			if os.path.isdir(self.dir_or_file):
			    self.directory = self.dir_or_file
			else:
			    self.directory, self.default = os.path.split(self.dir_or_file)
			    
		pattern = parse_default(self.pattern,self.var_type,self.command)
		self.set_filter(self.directory, pattern)
		default = parse_default(self.default,self.var_type,self.command)
		
		if not default or (default == 1):
			default = file
			
		self.set_selection(default)
		self.filter_command()
		self.selection.focus_set()
	
	def update_default(self):
		pass
	
	def get(self):
		files = self.get_selection()
		if self.multi_file:
			list_str = ''
			for file in files:
				if not self.path_file:
					file = os.path.split(file)[1]
				list_str = '%s\"%s\",' % (list_str, file)
			list_str = '(%s)' % list_str[0:(len(list_str)-1)]
			return self.var_name, list_str
		else:
			if not self.path_file:
					files = os.path.split(files)[1]
			files = '\"%s\"' % files
			return self.var_name, files

	def ok_event(self, event):
		self.ok_command()

	def ok_command(self, **kw):
		files = self.get_selection()
		if self.multi_file:
			file = files[0]
		else:
			file = files
			
		if self.key:
			directory, pattern = self.get_filter()
			if self.get():
				directory = os.path.dirname(self.get_first())
			dialogstates[self.key] = directory, file, pattern
		    
		# use the command manager
		self.command.manager.manager_ok(
			menu_title = self.command.menu_title)
	
	def cancel_command(self, **kw):
		# use the command manager
		self.command.manager.manager_cancel(
			menu_title = self.command.menu_title)

	def dirs_double_event(self, event):
		self.filter_command()

	def dirs_select_event(self, event):
		dir, pat = self.get_filter()
		subdir = self.dirs.get('active')
		dir = os.path.normpath(os.path.join(self.directory, subdir))
		self.set_filter(dir, pat)

	def files_double_event(self, event):
		self.ok_command()

	def files_select_event(self, event):
		file = self.files.get('active')
		self.set_selection(file)

	def filter_command(self, *args, **kw):
		dir, pat = self.get_filter()
		try:
		    names = os.listdir(dir)
		except os.error:
		    self.frame.winfo_toplevel().bell()
		    return
		self.directory = dir
		self.set_filter(dir, pat)
		names.sort()
		subdirs = [os.pardir]
		matchingfiles = []
		for name in names:
		    fullname = os.path.join(dir, name)
		    if os.path.isdir(fullname):
			subdirs.append(name)
		    elif fnmatch.fnmatch(name, pat):
			matchingfiles.append(name)
		self.dirs.delete(0, END)
		for name in subdirs:
		    self.dirs.insert(END, name)
		self.files.delete(0, END)
		for name in matchingfiles:
		    self.files.insert(END, name)
		head, tail = os.path.split(self.get_first())
		if tail == os.curdir: tail = ''
		self.set_selection(tail)

	def get_filter(self):
		filter = self.filter.get()
		filter = os.path.expanduser(filter)
		if filter[-1:] == os.sep or os.path.isdir(filter):
		    filter = os.path.join(filter, "*")
		return os.path.split(filter)
	
	def get_first(self):
		if self.multi_file:
			try:
			    return self.files.get(self.files.curselection()[0])
			except IndexError:
			    return ''
		else:
			return self.selection.get()
	    
	def get_selection(self):
		if self.multi_file:
			list = []
			selection = self.files.curselection()
			for index in selection:
				entry = self.files.get(index)
				list.append(os.path.expanduser(entry))
			return list
			
		file = self.selection.get()
		file = os.path.expanduser(file)
		return file

	def filter_it(self, pat=None, **kw):
		self.set_filter(pat=pat)
		self.filter_command()
	    
	def set_filter(self, dir=None, pat=None, **kw):
		dir = dir or self.directory
		if not os.path.isabs(dir):
		    try:
			pwd = os.getcwd()
		    except os.error:
			pwd = None
		    if pwd:
			dir = os.path.join(pwd, dir)
			dir = os.path.normpath(dir)
		self.filter.delete(0, END)
		self.filter.insert(END, os.path.join(dir or os.curdir, pat or "*"))

	def set_selection(self, file):
		self.selection.delete(0, END)
		self.selection.insert(END, os.path.join(self.directory, file))

class FOpen_Option(Base_File_Option):
	def ok_command(self, **kw):
		if self.multi_file:
			files = self.get_selection()
			for file in files:			
			    if not os.path.isfile(file):
				    self.widgets['main'].winfo_toplevel().bell()
				    return
			this_file = files[0]
		else:
			file = self.get_selection()
			this_file = file
			if not os.path.isfile(file):
				self.widgets['main'].winfo_toplevel().bell()
				return
		if self.key:
			directory, pattern = self.get_filter()
			if self.get():
			    directory = os.path.dirname(self.get_first())
			dialogstates[self.key] = directory, this_file, pattern
    
		# use the command manager
		self.command.manager.manager_ok(
			menu_title = self.command.menu_title)

class FSave_Option(Base_File_Option):
	title = "Save File Selection Dialog"

	def ok_command(self, **kw):
		file = self.get_selection()
		if os.path.exists(file):
		    if os.path.isdir(file):
			self.widgets['main'].winfo_toplevel().bell()
			return
		    d = Dialog(self.widgets['main'].winfo_toplevel(),
			       title="Overwrite Existing File Question",
			       text="Overwrite existing file %s?" % `file`,
			       bitmap='questhead',
			       default=1,
			       strings=("Yes", "Cancel"))
		    if d.num != 0:
			return
		else:
		    head, tail = os.path.split(file)
		    if not os.path.isdir(head):
			self.widgets['main'].winfo_toplevel().bell()
			return
		if self.key:
			directory, pattern = self.get_filter()
			if self.get_first():
			    directory = os.path.dirname(self.get_first())
			dialogstates[self.key] = directory, file, pattern
		    
		# use the command manager
		self.command.manager.manager_ok(
			menu_title = self.command.menu_title)

###############################################################
#
#	Option functions:
#		Functions called by options during parsing.
#
#
############################
def parse_default(default, vtype, command):
	# parse the default designator
	# which may be in the form of:
	#	value
	#	$variable
	#	$global_variable key
	
	# make things easier from inside tkCoLa
	parent = command.parent
	manager = command.manager
	
	if not default:
		return
		
	if not default[0] == '$':
		return default
	
	# cut out the $ designator
	default = default[1:]
	try:
		default = eval(default)
	except (NameError, AttributeError, KeyError, IndexError):
		try:
			default = global_vars[default]
		except KeyError:
			return None
	
	var_type = get_realtype(vtype)
	
	if type(default) != var_type:
		return 1
	
	if var_type in (IntType, FloatType):
		# make this into a string
		default = str(default)
		
	return default	
			
def get_realtype(type_string):
	# returns a Type
	if type_string in INT:
		return IntType
	if type_string in FLOAT:
		return FloatType
	if type_string in STRING:
		return StringType
	# list types
	type_string = string.split(type_string, '_')[0]
	if type_string in LIST:
		return TupleType
		
	print 'Command Error, parser: Unrecognized Type Designator'
	return None

def get_type(var_type):
	if not var_type:
		return
	split = string.split(var_type, '_')
	var_type = split[0]
	if var_type in INT:
		return 'int'
	if var_type in FLOAT:
		return 'float'
	if var_type in STRING:
		return 'string'
	if var_type in LIST:
		try:
			unit_type = get_type(split[1])
		except IndexError:
			return 'list'
			
		return ('list', unit_type)
		
	print 'Command Error, parser: Unrecognized Type Designator'
	return None
