#!/usr/local/bin/python

#	
#	
#	Graph: Graphing Framework v0.9
#	
#	Jason McDermott 1/22/2000, mcdermoj@ohsu.edu
#	
#	
#	

import os
import sys
import getopt
import string
import types
import math
import copy
import FileDialog

from Graph_Include import *
from Graph_Tags import *
import Commands
from Tkinter import *

SimpleType = 'simple'
ListIndexedType = 'list_indexed'
ListRecordType = 'list_record'
ReferenceType = 'reference'	
		
class Graph_Record:
	def __init__(self, legend=None, axes=None, label_x="",
			label_y="", datalist=[],
			parent=None, command_manager=None, 
			height=400, width=500, 
			grid_x=10, grid_y=10, bg_color='grey',
			graph_color='lightblue', graph_out='black',
			graph_out_width=1, symbol_size=0, edge_x=10, 
			edge_y=10, y_axis_width=100, x_axis_width=50,
			x_step=0, y_step=0,
			max_x='auto', max_y='auto', 
			min_x='auto', min_y='auto', 
			buffer_x=15, buffer_y=15,
			reference_x=None, reference_y=None, 
			reverse_x=0, reverse_y=0, 
			graph_frame=None, **kw):
	 
		self.legend = legend or Legend_Record()
		self.axes = axes or Axes_Record(graph=self, label_x=label_x,
				label_y=label_y)
		self.parent = parent
		self.command_manager = command_manager
		
		# pass these to the legend
		# command manager is required for 
		# some graph bindings
		self.legend.parent = parent
		self.legend.command_manager = command_manager
		self.legend.graph_record = self
		
		self.datalist = datalist
		self.height = height
		self.width = width
		self.bg_color = bg_color
		self.graph_color = graph_color
		self.graph_out = graph_out
		self.graph_out_width = graph_out_width
		self.symbol_size = symbol_size
		
		self.grid_x = grid_x
		self.grid_y = grid_y
		self.edge_x = edge_x
		self.edge_y = edge_y
		self.y_axis_width = y_axis_width
		self.x_axis_width = x_axis_width
		self.x_step = x_step
		self.y_step = y_step
		
		# if these aren't specified they will
		# be automatically calculated
		self._max_x = max_x
		self._max_y = max_y
		self._min_x = min_x
		self._min_y = min_y
		
		# buffers given in percentage of max_x or max_y
		self.buffer_x = float(buffer_x / 100.0)
		self.buffer_y = float(buffer_y / 100.0)
		
		# valid reference values are (strings)
		# left, right, top, bottom, 
		# center x and center y
		self.reference_x = reference_x
		self.reference_y = reference_y
		
		self.reverse_x = reverse_x
		self.reverse_y = reverse_y
		
		self._graph_frame = None

		self._init_keys(kw)
		
		if graph_frame:
			self._make_graph(graph_frame)
		
	def _init_keys(self, key_dict):
		# hook for handling type specific keywords
		# passed as a dictionary from **kw.
		pass
		
	def _init_graph(self, **kw):
		# place where data handling, graph options,
		# sizing, etc are performed
		# This will work for simple line or point
		# 	graphs but may need to be modified
		#	for others.
		
		self._get_axes_width()
		self._get_spread()
		self._get_dimensions()
		self._get_graph_box()
		self._get_maps()
		self._init_axes()
		
	def _init_axes(self, **kw):
		# initialize the axes
		x_step = self.x_step or self.spread_x / 8
		y_step = self.y_step or self.spread_y / 10
		
		self.axes.initialize(self._x_map, self._y_map, self.min_x,
					self.max_x, self.min_y, self.max_y,
					self.x_box, self.y_box,
					x_step, y_step)
					
	def _get_axes_width(self):
		# minor calculations relating to
		# widths of axes- this could be
		# fed into an axis class
		if self.y_axis_width < 0:
			# right
			self.y_axis_side = 1
		else:
			# default is left side
			self.y_axis_side = 0
			
		if self.x_axis_width < 0:
			# right
			self.x_axis_side = 1
		else:
			# default is left side
			self.x_axis_side = 0
		
		self.y_axis_width = abs(self.y_axis_width)
		self.x_axis_width = abs(self.x_axis_width)
		
	def _get_dimensions(self):
		# calculate the dimensions of the
		# total graphing area
		if self.width and self.spread_x:
			graph_width = self.width
			self.grid_x = self.width / self.spread_x
		elif not self.spread_x:
			graph_width = 10
			self.grid_x = 1
			
		else:
			graph_width = (self.grid_x * self.spread_x)
			
		self._paper_width = (graph_width + (self.edge_x * 2) +
					self.y_axis_width)
				 
		if self.height and self.spread_y:
			self._paper_height = self.height
			self.grid_y = self.height / self.spread_y
		elif not self.spread_y:
			self._paper_height = 10
			self.grid_y = 1
			
		else:
			self._paper_height = self.width * 0.75
		
		self.paper_box = (1, 1, self._paper_width, self._paper_height)
	
	def _get_spread(self):
		# Automatic calculation is specified with
		# by setting the initial parameter to 'auto'
		if type(self._max_x) == StringType:
			max_x = self.get_max_x()
		else:
			max_x = self._max_x
			
		if type(self._min_x) == StringType:
			min_x = self.get_min_x()
		else:
			min_x = self._min_x
			
		if type(self._max_y) == StringType:
			max_y = self.get_max_y()
		else:
			max_y = self._max_y
			
		if type(self._min_y) == StringType:
			min_y = self.get_min_y()
		else:
			min_y = self._min_y
		
		# add a percentage buffer
		self.max_x = max_x + (max_x * self.buffer_x)
		if min_x:
			self.min_x = min_x - (min_x * self.buffer_x)
		else:
			self.min_x = min_x
		
		self.max_y = max_y + (max_y * self.buffer_y)
		if min_y:
			self.min_y = min_y - (min_y * self.buffer_y)
		else:
			self.min_y = min_y
		
		if self.min_x < 0:
			if abs(self.min_x) > self.max_x:
				self.spread_x = abs(self.min_x) * 2
			else:
				self.spread_x = self.max_x * 2
		else:
			self.spread_x = self.max_x - self.min_x
			
		if self.min_y < 0:
			if abs(self.min_y) > self.max_y:
				self.spread_y = abs(self.min_y) * 2
			else:
				self.spread_y = self.max_y * 2
		else:
			self.spread_y = self.max_y - self.min_y
			
	def _get_graph_box(self):
		# define the graphing box
		if self.y_axis_side:
			# place axis on the right
			left = self.edge_x
			right = self._paper_width - self.edge_x - self.y_axis_width
		else:
			left = self.edge_x + self.y_axis_width
			right = self._paper_width - self.edge_x
		
		if self.x_axis_side:
			# place axis on the top
			top = self.edge_y + self.x_axis_width
			bottom = self._paper_height - self.edge_y
		else:
			top = self.edge_y
			bottom = self._paper_height - self.edge_y - self.x_axis_width 
			
		self.graph_box = (left, top, right, bottom)
				
		self.height = bottom - top
		self.width = right - left
		
		self.top = top
		self.bottom = bottom
		self.left = left
		self.right = right
		self.center_y = self.top + (self.height / 2)
		self.center_x = self.left + (self.width / 2)
		
		# define axes boxes
		self.x_box = (self.y_axis_width, bottom + 1, self.right-self.edge_x, 
				bottom + self.x_axis_width)
		self.y_box = (self.edge_x, self.edge_y, left, bottom)
		
	def _get_maps(self):
		# define the mapping parameters references and x/y maps
		# for straight x, y graphing
		if not self.reference_x:
			if (self.min_x < 0) and (self.max_x > 0):
				reference_x = 'center x'
			elif (self.min_x < 0) and (self.max_x < 0):
				reference_x = 'right'
				self.reverse_x = 1
			else:
				reference_x = 'left'
		else:
			reference_x = self.reference_x
			
		if not self.reference_y:
			if (self.min_y < 0) and (self.max_y > 0):
				reference_y = 'center y'
			elif (self.min_y < 0) and (self.max_y < 0):
				reference_y = 'top'
				self.reverse_y = 1
			else:
				reference_y = 'bottom'
		else:
			reference_y = self.reference_y
					
		self.ref_x = self._get_direction(reference_x, self.graph_box)
		self.ref_y = self._get_direction(reference_y, self.graph_box)
		
		self.ymap = self.height / self.spread_y
		self.xmap = self.width / self.spread_x
		
	def _get_direction(self, reference, (left, top, right, bottom)):
		if reference in ('l', 'L', 'left', 'LEFT'):
			return left
		elif reference in ('r', 'R', 'right', 'RIGHT'):
			return right
		elif reference in ('t', 'T', 'top', 'TOP'):
			return top
		elif reference in ('b', 'B', 'bottom', 'BOTTOM'):
			return bottom
		elif reference in ('cx', 'CX', 'center x', 'center_x', 'CENTER X'):
			return ((right - left) / 2) + left
		elif reference in ('cy', 'CY', 'center y', 'center_y', 'CENTER Y'):
			return ((bottom - top) / 2) + top
		else:
			return None
		
	def _make_graph(self, frame=None, **kw):
		## how do we set up the graphing area when we're given a frame
		self._init_graph()
		self._graph_frame = frame
		self._paper = Canvas(frame, height=self._paper_height,
						width=self._paper_width)
		self._paper.pack()
		self._update()
		
	def _deep_update(self, **kw):
		self._get_spread()
		self._get_maps()
		self._init_axes()
		self._update()
			
	def _update(self, **kw):
		self._clear_paper()
		self._back_paper()
		self._graph()
		self._fore_paper()
		self._legend()
		self.axes.draw(self._paper)
		
	def _update_colors(self, tags=None, colors=None, **kw):
		if type(tags) == StringType:
			tags = (tags,)
			colors = (colors,)
		if type(tags) in (ListType, TupleType):
			i = 0
			for tag in tags:
				item = canvas.find_withtag(tag)
				canvas.itemconfigure(fill=colors[i])
				i = i + 1

	def _clear_paper(self, **kw):
		## how do we clear off the drawing area
		self._paper.delete(ALL)
			    	
	def _back_paper(self, **kw):
		## how do we draw the background portion of
		## the graphing 'paper'; the graph, legend
		## and foreground paper portion (axes?) 
		## will overlay this portion of the paper
		paper = self._paper
		width = self._paper_width
		height = self._paper_height
		
		# make the overall background rectangle
		paper.create_rectangle(self.paper_box, fill=self.bg_color,
						     outline='', tags=GRAPH_BG_TAG)
		
		# make the actual graphing surface
		paper.create_rectangle(self.graph_box, fill=self.graph_color,
					outline=self.graph_out, width=self.graph_out_width,
					tags=GRAPH_FG_TAG)
						     
	def _graph(self, **kw):
		## how do we draw the graph itself?
		for graph_entry in self.datalist:
			graph_entry.draw(self._paper, self._x_map, self._y_map, 
							width=self.symbol_size)
			
	def _x_map(self, x):
		## calculate the actual x value from the given value
		## in this case using the mapping ratio xmap
		if self.min_x < 0:
			min_x = 0
		else:
			min_x = self.min_x
			
		if self.reverse_x:
			return  self.ref_x - ((x - min_x) * self.xmap)
		else:
			return  self.ref_x + ((x - min_x) * self.xmap)
		
	def _y_map(self, y):
		## calculate the actual y value from the given value
		## in this case using the mapping ratio ymap
		## note that the convention for ymap is reversed
		## from xmap since the y should be graphed
		## with up being up by default
		if self.min_y < 0:
			min_y = 0
		else:
			min_y = self.min_y
			
		if self.reverse_y:
			return  self.ref_y + ((y - min_y) * self.ymap)
		else:
			return  self.ref_y - ((y - min_y) * self.ymap)
			
	def _x_dist(self, x):
		## calculate un-referenced value of x
		return x * self.xmap
	def _y_dist(self, y):
		## calculate un-referenced value of y
		return y * self.ymap
		
	def _fore_paper(self, **kw):
		## drawing of axes and anything else
		## we want to lay on top of everything
		## except the legend (if present)
		
		# re-assert the box around the graph
		paper = self._paper
		paper.create_rectangle(self.paper_box, fill='', outline=self.graph_out,
					width=self.graph_out_width, tags=GRAPH_FGBOX_TAG)
		
	def _legend(self, **kw):
		## use the legend record to draw it unless
		## this method is overridden.
		self.legend._draw_legend(self._paper)
		
	def _save_paper(self, filename=None, all_data=FALSE, **kw):
		## dump the graph paper to a postscript file
		if not filename:
			return
		
		if all_data:
			for i in range(len(self.datalist)):
				self._select_member(index=i)
				fname = string.split(filename, '.')[0]
				fname = "%s_%03d.ps" % (fname, i)
				self._save_paper(filename=fname, all_data=FALSE)
			return
					
		output = self._get_postscript()
		
		try:
			fd = open(filename, 'w')
			fd.write(output)
			fd.close()
		except IOError:
			print "Graph IO Error: Could not write file %s" % filename
			return
		
	def _get_postscript(self, **kw):
		return self._paper.postscript()
		
	def _select_member(self, index=0, all=0, **kw):
		for record in self.datalist:
			record.in_graph = all
			
		if type(index) == IntType:
			self.datalist[index].in_graph = 1
		elif type(index) == TupleType:
			for n in index:
				self.datalist[n].in_graph = 1
		self._update()
	
	def set_datalist(self, dict=None, **kw):
		kw = dict or kw
		n = 0
		for record in self.datalist:
			for key in kw.keys():
				if type(kw[key]) in (ListType, TupleType):
				    if record.__dict__.has_key(key):
				    	try:
					    record.__dict__[key] = kw[key][n]
					except IndexError:
					    pass
			n = n + 1
				
	def get_max_x(self):
		max_x = -10000.0
		for rec in self.datalist:
			rec_max = rec.get_max_x()
			if rec_max > max_x:
				max_x = rec_max
		return max_x
		
	def get_max_y(self):
		max_y = -10000.0
		for rec in self.datalist:
			rec_max = rec.get_max_y()
			if rec_max > max_y:
				max_y = rec_max
		return max_y
		
	def get_min_x(self):
		min_x = 100000.0
		for rec in self.datalist:
			rec_min = rec.get_min_x()
			if rec_min < min_x:
				min_x = rec_min
		return min_x
		
	def get_min_y(self):
		min_y = 100000.0
		for rec in self.datalist:
			rec_min = rec.get_min_y()
			if rec_min < min_y:
				min_y = rec_min
		return min_y

##############################################################################
# SubClass:	MultiPane_Graph
##
#	This is a special case of graph record which allows more than
#	one graph pane to be displayed in and controlled by the same
#	graph_window TkCoLa Command window.
#
#	This record is considered the 'master' and a list of 'slave'
#	graph records is provided upon initialization. Updating the
#	master will update the slaves, etc.
#

class MultiPane_Graph(Graph_Record):
	def __init__(self, legend=None, axes=None, datalist=[],
			parent=None, command_manager=None, 
			height=400, width=500, 
			grid_x=10, grid_y=10, bg_color='grey',
			graph_color='lightblue', graph_out='black',
			graph_out_width=1, symbol_size=0, edge_x=10, 
			edge_y=10, y_axis_width=100, x_axis_width=50,
			x_step=0, y_step=0,
			max_x='auto', max_y='auto', 
			min_x='auto', min_y='auto', 
			buffer_x=15, buffer_y=15,
			reference_x=None, reference_y=None, 
			reverse_x=0, reverse_y=0,
			graph_frame=None, slave_list=None, **kw):
	 
		self.legend = legend or Legend_Record()
		self.axes = axes or Axes_Record(graph=self)
		self.parent = parent
		self.command_manager = command_manager
		
		# pass these to the legend
		# command manager is required for 
		# some graph bindings
		self.legend.parent = parent
		self.legend.command_manager = command_manager
		self.legend.graph_record = self
		
		self.datalist = datalist
		self.height = height
		self.width = width
		self.bg_color = bg_color
		self.graph_color = graph_color
		self.graph_out = graph_out
		self.graph_out_width = graph_out_width
		self.symbol_size = symbol_size
		
		self.grid_x = grid_x
		self.grid_y = grid_y
		self.edge_x = edge_x
		self.edge_y = edge_y
		self.y_axis_width = y_axis_width
		self.x_axis_width = x_axis_width
		self.x_step = x_step
		self.y_step = y_step
		
		# if these aren't specified they will
		# be automatically calculated
		self._max_x = max_x
		self._max_y = max_y
		self._min_x = min_x
		self._min_y = min_y
		
		# buffers given in percentage of max_x or max_y
		self.buffer_x = float(buffer_x / 100.0)
		self.buffer_y = float(buffer_y / 100.0)
		
		# valid reference values are (strings)
		# left, right, top, bottom, 
		# center x and center y
		self.reference_x = reference_x
		self.reference_y = reference_y
		
		self.reverse_x = reverse_x
		self.reverse_y = reverse_y
		
		self._init_keys(kw)
		
		self.slave_list = []
		
		# we include this record as the first 'slave'
		#	so that it can update itself, etc.
		self.slave_list.append(self)
		
		if type(slave_list) in (TupleType, ListType):
			for slave in slave_list:
				self.slave_list.append(slave)
		elif type(slave_list) == InstanceType:
			self.slave_list.append(slave_list)
		
		self.graph_frame = graph_frame
		if graph_frame:
			self._make_graph(graph_frame)
		
	def _m_make_graph(self, frame=None, **kw):
		# the user may supply the frame as a list (to specify
		#	locations of the individual graphs
		if type(frame) in (TupleType, ListType):
			if len(frame) == len(self.slave_list):
				for i in range(len(frame)):
					self.slave_list[i]._make_graph(frame[i])
				return
		else:
			self.graph_frame = frame
			for i in range(len(self.slave_list)):
				sub = Frame(frame)
				sub.pack(side=TOP, fill=BOTH, expand=1)
				self.slave_list[i]._make_graph(sub)
			
	def _m_deep_update(self, **kw):
		for graph in self.slave_list:
			graph._deep_update()
		
	def _m_update(self, **kw):
		for graph in self.slave_list:
			graph._update()
			
	def _save_paper(self, filename=None, all_data=FALSE, **kw):
		## dump the graph paper to a postscript file
		if not filename:
			return
		
		if all_data:
			for i in range(len(self.datalist)):
				for graph in self.slave_list:
					graph._select_member(index=i)
				fname = string.split(filename, '.')[0]
				fname = "%s_%03d.ps" % (fname, i)
				self._save_paper(filename=fname, all_data=FALSE)
			return
					
		output = self._get_postscript()
		
		try:
			fd = open(filename, 'w')
			fd.write(output)
			fd.close()
		except IOError:
			print "Graph IO Error: Could not write file %s" % filename
			return
			
	def _get_postscript(self, **kw):
		postscript = []
		pagey = 0
		for graph in self.slave_list:
			postscript.append(graph._paper.postscript(pagex=0, pagey=pagey, 
					pageanchor='nw', pageheight='8c'))
			pagey = pagey + graph._paper_height + 10
		
		# slurp into one valid postscript file
		output = ''
		i = 1
		one = 0
		for post in postscript:
			# remove clipping
			post = string.replace(post, 'closepath clip newpath', 'closepath newpath')
			if one:
				# remove header of postscript (not first one)
				index = string.find(post, '\nsave')
				real = string.find(post, 'scale', index) + 6
				post = post[real:]
				
			# truncate files at end, saving end section for re-attachment
			index = string.find(post, '\nrestore showpage')
			end = post[index:]
			post = post[0:index]
			output = "%s\n\n\n%s" % (output, post)
			one = 1
			i = i + 1
		
		output = "%s\n%s" % (output, end)
		return output
	
	def _m_select_member(self, index=0, all=0, **kw):
		for graph in self.slave_list:
			# DANGER: if the slave graphs don't have the
			#	  same number of data entries as the
			#	  parent (this) graph will cause 
			#	  problems!
			graph._select_member(index=index, all=all)
	
	def m_set_datalist(self, **kw):
		for graph in self.slave_list:
			graph.set_datalist(kw)
			
	def add_graph_record(self, graph_record=None, frame=None, **kw):
		self.slave_list.append(graph_record)
		
		if not frame and self.graph_frame:
			sub = Frame(self.graph_frame)
			sub.pack(side=TOP, fill=BOTH, expand=1)
			graph_record._make_graph(sub)
		else:
			graph_record._make_graph(frame)
		
class Graph_Entry:
	def __init__(self, title='', label='', color=None, symbol=None, s_size=4, in_legend=1,
			in_graph=1, x=0, y=0, z=0, x_error=0, y_error=0, z_error=0, 
			map_x=None, map_y=None, map_z=None, ffA=1, ffB=1, ffC=1,
			reclist=[], data=None):
			
		self.title = title
		self.label = label
		self.color = color
		self.symbol = symbol or self.get_symbol()
		self.s_size = s_size
		self.in_legend = in_legend
		self.in_graph = in_graph
		
		# this list can be a graph_entry list
		# to allow for multiple data points
		# with the same characteristics (color, etc.)
		# and/or continuity (lines)
		self.reclist = reclist
		
		# data can be a record from which data is derived
		# mapping to data attributes can be managed by
		# use of the set_x, set_y and set_z functions.
		# map_x, map_y and map_z should be strings
		# denoting the name of the attribute to 
		# map the particular coordinate to.
		self.type = self.get_mapping(data, map_x, map_y, map_z)
		if not self.type:
			return
				
		self.x = x
		self.y = y
		self.z = z
		self.x_error = x_error
		self.y_error = y_error
		self.z_error = z_error
		
		self._filter_flag_A = ffA
		self._filter_flag_B = ffB
		self._filter_flag_C = ffC
	
	def get_symbol(self):
		return Box_Symbol()
	
	# this draw method takes the paper and two
	# mapping methods for x and y as argument.
	# If there is a reclist associated with this
	# entry then that is drawn. Data list is handled
	# appropriately if it is a list.
	def draw(self, paper, x_map, y_map, color=None, width=None):
		if not self.in_graph:
			return
			
		color = color or self.color
		width = width or self.s_size
		
		# graph record list takes precedence
		if self.reclist:	
		    for rec in self.reclist:
		    	if self._filter_hook_A(rec):
		    		rec.draw(paper, x_map, y_map, color=color, width=width)
		
		# a list type?	
		elif self.type[0] == 'l':
			# is the data a list of records?
			if self.type == ListRecordType:
				for member in self.data:
					if self._filter_hook_B(member):
					    x = x_map(self.get_x(member))
					    y = y_map(self.get_y(member))
					    self.symbol.draw(paper, x, y, size=width, color=color)	
			
			elif self.type == ListIndexedType:		
				for index in range(0, len(self.data[self.map_x])):
					if self._filter_hook_B(None):
					    x = x_map(self.data[self.map_x][index])
					    y = y_map(self.data[self.map_y][index])
					    self.symbol.draw(paper, x, y, size=width, color=color)
					    	
		elif self._filter_hook_C():
		    x = x_map(self.get_x())
		    y = y_map(self.get_y())
		    self.symbol.draw(paper, x, y, size=width, color=color)
		    
		# else dont plot nothing
	
	# Filter flag hooks: these can be specified
	# as 1 or 0 in the init, specified as a 
	# callback, or replaced in a subclass.
	def _filter_hook_A(self, record=None):
		# filter for drawing the reclist
		if not record:
			return 1
		if type(self._filter_flag_A) in (FunctionType, MethodType):
			return self._filter_flag_A(record)
		return self._filter_flag_A
		
	def _filter_hook_B(self, record=None):
		# filter for drawing the data
		if not record:
			return 1
		if type(self._filter_flag_B) in (FunctionType, MethodType):
			return self._filter_flag_B(record)
		return self._filter_flag_B
		
	def _filter_hook_C(self, record=None):
		# filter for drawing individual data points
		if not record:
			return 1
		if type(self._filter_flag_C) in (FunctionType, MethodType):
			return self._filter_flag_C(record)
		return self._filter_flag_C
	
	def get_mapping(self, data, map_x, map_y, map_z):
		if not data:
			self.data = self.map_x = self.map_y = self.map_z = None
			return SimpleType
		
		self.data = data
		
		if type(data) in (ListType, TupleType):
			# list of records
			if type(data[0]) == InstanceType:
				this_type = ListRecordType
			elif type(data[0]) in (ListType, TupleType):
				this_type = ListIndexedType
				if (type(map_x) != IntType) or (type(map_y) != IntType):
					map_x = 1
					map_y = 2
					# map_z = 3
					
			self.map_x = self.get_map(map_x, this_type)
			self.map_y = self.get_map(map_y, this_type)
			
			if map_z:
				self.map_z = self.get_map(map_z, this_type)
			else:
				self.map_z = None
			if (self.map_x < 0) or (self.map_y < 0) or (self.map_z < 0):
				return None
					
		elif type(data) == InstanceType:
			this_type = ReferenceType
			self.map_x = self.get_map(map_x, this_type)
			self.map_y = self.get_map(map_y, this_type)
			if map_z:
				self.map_z = self.get_map(map_z, this_type)
			else:
				self.map_z = None
			if (self.map_x < 0) or (self.map_y < 0) or (self.map_z < 0):
				return None
			
		else:
			print 'Graph Error, data: unsupported data type %s' % str(type(data))
			self.data = self.map_x = self.map_y = self.map_z = None
			return
		
		return this_type
	
	# check to make sure mapping is OK		
	def get_map(self, attribute, this_type):
		if this_type == ListIndexedType:
			# must be given as attribute + 1 to denote the presence of something
			try:
				attribute = attribute - 1
				attempt = self.data[attribute]
			except (IndexError, TypeError):
				print 'Graph Error, data: data index %s not present' % str(attribute)
				return -1
				
			return attribute
		
		if this_type == ListRecordType:
			if self.data[0].__dict__.has_key(attribute):
				return attribute
			else:
				print 'Graph Error, data: attribute %s not found' % attribute
				return -1
				
		if this_type == ReferenceType:
			if self.data.__dict__.has_key(attribute):
				return attribute
			else:
				print 'Graph Error, data: attribute %s not found' % attribute
				return -1
				
	def get_x(self, data=None):
		if self.map_x:
			data = data or self.data
			return data.__dict__[self.map_x]
		else:
			return self.x
	def get_y(self, data=None):
		if self.map_y:
			data = data or self.data
			return data.__dict__[self.map_y]
		else:
			return self.y
	def get_z(self, data=None):
		if self.map_z:
			data = data or self.data
			return data.__dict__[self.map_z]
		else:
			return self.z
			
	def get_max_x(self):
		if self.reclist:
			max_x = -100000000.0
			for rec in self.reclist:
				maxx = rec.get_max_x()
				if maxx > max_x:
					max_x = maxx
			return max_x
			
		if self.type == ListRecordType:
			max_x = -100000000.0
			for rec in self.data:
				if self.get_x(rec) > max_x:
					max_x = self.get_x(rec)
			return max_x
		
		elif self.type == ListIndexedType:
			return max(self.data[self.map_x])
			
		return self.get_x()
		
	def get_max_y(self):
		if self.reclist:
			max_y = -100000000.0
			for rec in self.reclist:
				maxy = rec.get_max_y()
				if  maxy > max_y:
					max_y = maxy
			return max_y
			
		if self.type == ListRecordType:
			max_y = -100000000.0
			for rec in self.data:
				if self.get_y(rec) > max_y:
					max_y = self.get_y(rec)
			return max_y
		
		elif self.type == ListIndexedType:
			return max(self.data[self.map_y])
			
		return self.get_y()
		
	def get_min_x(self):
		if self.reclist:
			min_x = 100000000.0
			for rec in self.reclist:
				minx = rec.get_min_x()
				if minx < min_x:
					min_x = minx
			return min_x
			
		if self.type == ListRecordType:
			min_x = 100000000.0
			for rec in self.data:
				if self.get_x(rec) < min_x:
					min_x = self.get_x(rec)
			return min_x
		
		elif self.type == ListIndexedType:
			return min(self.data[self.map_x])
			
		return self.get_x()
		
	def get_min_y(self):
		if self.reclist:
			min_y = 100000000.0
			for rec in self.reclist:
				miny = rec.get_min_y()
				if  miny < min_y:
					min_y = miny
			return min_y
			
		if self.type == ListRecordType:
			min_y = 100000000.0
			for rec in self.data:
				if self.get_y(rec) < min_y:
					min_y = self.get_y(rec)
			return min_y
		
		elif self.type == ListIndexedType:
			return min(self.data[self.map_y])
			
		return self.get_y()

class Axes_Record:
	def __init__(self, x_axis=None, y_axis=None, 
			background=None, foreground=None,
			graph=None, label_x="", label_y="", **kw):
		# x axis record
		self.x_axis = x_axis or X_Axis(label=label_x)
		
		# y axis record
		self.y_axis = y_axis or Y_Axis(label=label_y)
		
		# background of graph
		self.background = background or Background()
		
		# foreground of graph (i.e. over the data)
		self.foreground = foreground
		
		# parent graph
		self.graph = graph
		
	def initialize(self, x_map_func, y_map_func, 
			xmin, xmax, ymin, ymax, 
			x_box, y_box, x_step, y_step, **kw):
		self.x_axis.initialize(x_map_func, xmin, xmax, x_box, x_step)
		self.y_axis.initialize(y_map_func, ymin, ymax, y_box, y_step)
		
	def draw(self, paper, **kw):
		self.x_axis.draw(paper)
		self.y_axis.draw(paper)

class Axis:
	def __init__(self, label="", **kw):
		self.label = label or ""
		
class X_Axis(Axis):
	def initialize(self, x_map_func, xmin, xmax, x_box, x_step=None, **kw):
		self.x_map = x_map_func
		self.xmin = xmin
		self.xmax = xmax
		self.x_box = x_box
		self.x_step = x_step
		
	def draw(self, paper, bound_box=None, color='black', **kw):
		def plot_tick(paper, y0, y1, x, x0, color, font=bold):
			paper.create_line(x0, y0, x0, y1, width=2, fill=color, 
						tags=AXIS_X_LINE_TAG)
			text = get_marker(x)
			paper.create_text(x0, (y1+7), text=text, font=font, 
					fill=color, anchor='center',
						tags=AXIS_X_TEXT_TAG)
					
		bound_box = bound_box or self.x_box
		y0 = bound_box[T0P]
		y1 = y0 + 6
		
		xmin = self.xmin
		xmax = self.xmax
		x_step = self.x_step
		
		stop = 0
		if xmin > 0:
			x = xmin
		else:
			x = 0.0
		while not stop:
			if x > xmax:
				stop = 1
			else:
				x0 = self.x_map(x)
				plot_tick(paper, y0, y1, x, x0, color)
				x = x + x_step
		
		x = float(-x_step)		
		while stop:
			if x < xmin:
				stop = 0	    
			else:
				x0 = self.x_map(x)
				plot_tick(paper, y0, y1, x, x0, color)
				x = x - x_step

		self.make_label(paper, bound_box)
		
	def make_label(self, paper, bound_box=None, font=boldb, color='black', **kw):
		if not self.label:
			return
			
		bound_box = bound_box or self.x_box
		y = bound_box[T0P] + 30
		x = int(((bound_box[RGT] - bound_box[LFT]) / 2) + bound_box[LFT])
		
		paper.create_text(x, y, text=self.label, font=font,
			fill=color, tags=AXIS_X_TEXT_TAG)	
											
class Y_Axis(Axis):	
	def initialize(self, y_map_func, ymin, ymax, y_box, y_step, **kw):
		self.y_map = y_map_func
		self.ymin = ymin
		self.ymax = ymax
		self.y_box = y_box
		self.y_step = y_step
	
	def draw(self, paper, bound_box=None, color='black', **kw):
		def plot_tick(paper, y, y0, x0, x1, color, font=bold):
			paper.create_line(x0, y0, x1, y0, width=2, fill=color,
						tags=AXIS_Y_LINE_TAG)
			text = get_marker(y)
			paper.create_text(x0, y0, text=text, font=font,
						fill=color, anchor='e',
						tags=AXIS_Y_TEXT_TAG)
						
		bound_box = bound_box or self.y_box
		x0 = bound_box[RGT] - 6
		x1 = bound_box[RGT]
		
		ymin = self.ymin
		ymax = self.ymax
		
		y_step = self.y_step
		
		stop = 0
		if ymin > 0:
			y = ymin
		else:
			y = 0.0
		while not stop:
			if y > ymax:
				stop = 1
			y0 = self.y_map(y)
			plot_tick(paper, y, y0, x0, x1, color)
			y = y + y_step
		
		if ymin < 0:		
			y = float(-y_step)		
			while stop:
				if y < ymin:
					stop = 0
				y0 = self.y_map(y)
				plot_tick(paper, y, y0, x0, x1, color)
				y = y - y_step
			
		self.make_label(paper, bound_box)
		
	def make_label(self, paper, bound_box=None, font=boldb, color='black', **kw):
		if not self.label:
			return
			
		bound_box = bound_box or self.y_box
		x = bound_box[LFT] + 8
		y = bound_box[T0P] + 15
		
		label = ""
		for c in self.label:
			label = "%s%s\n" % (label, c)
					
		paper.create_text(x, y, text=label, font=font,
			fill=color, anchor="n", tags=AXIS_Y_TEXT_TAG)			
		
# A do nothing Axis Class

class Null_Axis:
	def __init__(self, **kw):
		pass		
	def initialize(self, *args, **kw):
		pass
	def draw(self, *args, **kw):
		pass

		
def get_marker(numb, float_digits=2):
	if type(numb) == IntType:
		return ('%d' % numb)
	else:
		# eval_string = '\'\%f.%d\'\% numb' % float_digits
		return  ('%4.2f' % numb)
		
class Background:
	def __init__(self, **kw):
		pass

class Foreground:
	def __init__(self, **kw):
		pass
		
class Legend_Record:
	def __init__(self, x_loc=40, y_loc=40, margin=10,
			symbol_size=7, box_color='black',
			box_width=1, bg_color=None, bg_box_width=1,
			bg_outline='black', title_space_x=8,
			title_space_y=3, title='', header='Legend',
			font=bold, font_color='black', hide=0):
		
		self.x_loc = x_loc
		self.y_loc = y_loc
		
		self.margin = margin
		self.symbol_size = symbol_size
		self.box_color = box_color
		self.bg_color = bg_color
		self.bg_box_width = bg_box_width
		self.bg_outline = bg_outline
		self.title_space_x = title_space_x
		self.title_space_y = title_space_y
		
		self.title = title
		self.header = header
		self.font = font
		self.font_color = font_color
		self.hide = hide
		
		# we'll get these from the graph_record
		self.parent = None
		self.command_manager = None
		self.graph_record = None
		
	def _draw_legend(self, paper, ref_x=0, ref_y=0):
		# an example for working with simple graph
		# data such as a bar graph where each datalist
		# member gets its own legend entry
		
		if self.hide:
			return
			
		self.bg_color = self.bg_color or ''
		self.bg_outline = self.bg_outline or ''
		self.font_color = self.font_color or ''
		
		lx = self.x_loc + ref_x
		ly = self.y_loc + ref_y
		
		nx = lx + self.margin
		ny = ly + self.margin
		
		# to modify after the legend is drawn
		back_box = (lx, ly, lx, ly)
		background = paper.create_rectangle(back_box, fill=self.bg_color,
					outline=self.bg_outline, 
					width=self.bg_box_width,
					tags=LEGEND_BOX_TAG)
		
		paper.tag_bind('legend', '<B1-Motion>', self.move_legend)
					
		if self.command_manager:
			# bind mouse double to open a command window
			paper.tag_bind('legend', '<Double-Button-1>', 
				(lambda event=None, self=self:
				self.command_manager.manager_window(command_title='Legend Options', 
				window_config='def', data=self.graph_record)))
		
		if self.title:
			paper.create_text(nx, ly-20, text=self.title, 
					anchor='nw', font=self.font,
					fill=self.font_color, tags=LEGEND_TEXT_TAG)
		
		max_width = self.margin
		
		if self.header:
			t = paper.create_text(nx, ny, text=self.header, 
					anchor='nw', font=self.font,
					fill=self.font_color, tags=LEGEND_TEXT_TAG)
					
			w, h = get_dims(paper, t)
			if w > max_width:
				max_width = w
				
			ny = ny + h + self.title_space_y
			
		for rec in self.graph_record.datalist:
			if not rec.in_legend or not rec.in_graph:
				continue
			
			color = rec.color	
			if not color:
				color = ''
				# get the color of the first
				# reclist member for the entry
				if rec.reclist:
					color = rec.reclist[0].color
				
			rec.symbol.legend(paper, nx, ny, self.symbol_size,
						color=color)
			x = self.symbol_size + self.title_space_x
			
			title = rec.title
			if not title:
				title = ''
				if rec.reclist:
					title = rec.reclist[0].title
					
			t = paper.create_text(nx+x, ny - (self.symbol_size/2), 
					text=title, anchor='nw',
					font=self.font, fill=self.font_color,
					tags=LEGEND_TEXT_TAG)
					
			w, h = get_dims(paper, t)
			if (w+x) > max_width:
				max_width = w+x
				
			ny = ny + max(self.symbol_size, h) + self.title_space_y
			
		paper.coords(background, lx, ly, nx+max_width+self.margin, ny)
		
	def move_legend(self, event=None):
		canvas = self.graph_record._paper
		delta_x = self.x_loc - event.x
		delta_y = self.y_loc - event.y
		self.x_loc = event.x
		self.y_loc = event.y

		for item in canvas.find_withtag("legend"):
			if canvas.type(item) == "rectangle":
				x,y,x1,y1 = canvas.coords(item)
				new_x = x - delta_x
				new_y = y - delta_y
				new_x1 = x1 - delta_x
				new_y1 = y1 - delta_y
				canvas.coords(item, new_x, new_y, new_x1, new_y1)
			else:
				x,y = canvas.coords(item)
				new_x = x - delta_x
				new_y = y - delta_y
				canvas.coords(item, new_x, new_y)

def get_dims(paper, item):
	box = paper.bbox(item)
	height = box[3]-box[1]
	width = box[2]-box[0]
	return width, height

# Base Symbol Drawing Class for Graphs		
class Symbol:
	def __init__(self):
		pass
		
	def draw(self, paper, x, y, size=0, color='', tag=None):
		# draw the graph symbol
		pass
	
	def legend(self, paper, x, y, size=0, color=''):
		x = x + (size / 2)
		y = y + (size / 2)
		self.draw(paper, x, y, size=size, color=color, tag='legend')
		
class Box_Symbol(Symbol):
	def draw(self, paper, x, y, x1=0, y1=0,
			size=0, color='black',
			line_width=1, line_color='black', tag=''):
		half = size / 2
		x1 = x1 or (x+half)
		y1 = y1 or (y+half)
		plot_box = (x-half, y-half, x1, y1)
		paper.create_rectangle(plot_box, fill=color, width=line_width,
			    			outline=line_color, tags=tag)

class Dot_Symbol(Symbol):
	def draw(self, paper, x, y, x1=0, y1=0,
			size=0, color='black',
			line_width=1, line_color='black',
			tag = ''):
		half = size/2
		x1 = x1 or (x+half)
		y1 = y1 or (y+half)
		plot_box = (x-half, y-half, x1, y1)
		paper.create_oval(plot_box, fill=color, width=line_width,
			    			outline=line_color, tags=tag)
						
class Text_Symbol(Symbol):
	def __init__(self, char=None, font=None):
		self.char = char or 'X'
		if len(self.char) > 1:
			self.char = self.char[0]
		self.font = font or bold
		
	def draw(self, paper, x, y, size=0, color='', tag=''):
		paper.create_text(x, y, text=self.char, 
					anchor='center', font=self.font,
					fill=color, tags=tag)
					
	def legend(self, paper, x, y, size=0, color=''):
		self.draw(paper, x, y, size=size, color=color, tag='legend')
					
## routine to make datalists from a record list
## and x and y mapped attributes. att_x and att_y
## should be the names (as strings) of the
## attributes of record in rec_list that will
## be mapped to the x and y coordinates
def make_datalist(rec_list, att_x, att_y, symbol=Box_Symbol(), **kw):
	if type(rec_list) not in (ListType, DictType):
		print 'make_datalist error: variable not a list or dict'
		return
	if type(att_x) != StringType or type(att_y) != StringType:
		print 'make_datlist error: x and y map need to be strings'
		return
		
	datalist = []
		
	if type(rec_list) == ListType:
	    for rec in rec_list:
		if rec.__dict__.has_key(att_x) and rec.__dict__.has_key(att_y):
			entry = Graph_Entry(map_x=att_x, map_y=att_y, data=rec, symbol=symbol)
			datalist.append(entry)
		else:
			print 'make_datalist error: attribute not found in record'
			break	
	else:
	    for key in rec_list.keys():
		rec = rec_list[key]
		if rec.__dict__.has_key(att_x) and rec.__dict__.has_key(att_y):
			entry = Graph_Entry(map_x=att_x, map_y=att_y, data=rec, symbol=symbol)
			datalist.append(entry)
		else:
			print 'make_datalist error: attribute not found in record'
			break
			
	return datalist	
	
## Routine to take a keyword dictionary and
## return the value or a default value for
## the particular key specified. key can
## be a string or a tuple of strings.
def key_process(key_dict, keys, default=None):
	if type(keys) == StringType:
		if key_dict.has_key(keys):
			return key_dict[keys]
		else:	
			if default:
				return default
			else:
				print 'key_process error: key not present'
				return
				
	elif type(keys) == TupleType:
		ret = 0
		for key in keys:
			if key_dict.has_key(key):
				ret = key_dict[key]
				break
		if ret:
			return ret
		if default:
			return default
		else:
			print 'key_process error: key not present'
			return	
					
## Implementation of a Bar Graph class set
class Bar_Graph(Graph_Record):
	def _init_keys(self, key_dict):
		self.bar_width = key_process(key_dict, ('bar_width', 'bw'), 40)
		self.bar_spacing = key_process(key_dict, ('bar_spacing', 'spacing'), 10)
		self.axes = Bar_Axes(graph=self)
		
	def _get_dimensions(self):
		graph_width =((self.bar_width * self.max_x) +
				(self.bar_spacing * (self.max_x + 1)))
		self._paper_width = (graph_width + (self.edge_x * 2) +
					self.x_axis_width)
				 
		if self.height:
			self._paper_height = self.height
		else:
			self._paper_height = self.width * 0.75
		
		self.paper_box = (1, 1, self._paper_width, self._paper_height)	
		
	def _get_spread(self):
		# Automatic calculation is specified with
		# by setting the initial parameter to 'auto'
		if type(self._max_x) == StringType:
			self.max_x = len(self.datalist)
		else:
			self.max_x =  self._max_x
			
		if type(self._max_y) == StringType:
			max_y = self.get_max_y()
		else:
			max_y = self._max_y
			
		if type(self._min_y) == StringType:
			min_y = self.get_min_y()
		else:
			min_y = self._min_y
		
		# add a percentage buffer
		self.max_y = max_y + (max_y * self.buffer_y)
		self.min_y = min_y + (min_y * self.buffer_y)
		
		self.spread_x = self.max_x
			
		if self.min_y:
			if abs(self.min_y) > self.max_y:
				self.spread_y = abs(self.min_y) * 2
			else:
				self.spread_y = self.max_y * 2
		else:
			self.spread_y = self.max_y

	def _get_maps(self):
		# define the mapping parameters references and x/y maps
		# for straight x, y graphing
			
		if not self.reference_x:
			reference_x = 'left'
		else:
			reference_x = self.reference_x
		
		if not self.reference_y:
			if (self.min_y < 0) and (self.max_y > 0):
				reference_y = 'center y'
			elif (self.min_y < 0) and (self.max_y < 0):
				reference_y = 'top'
				self.reverse_y = 1
			else:
				reference_y = 'bottom'
		else:
			reference_y = self.reference_y
			
		self.ref_x = self._get_direction(reference_x, self.graph_box)
		self.ref_y = self._get_direction(reference_y, self.graph_box)
		
		self.ymap = self.height / self.max_y
		self.xmap = (self.width - self.bar_spacing) / self.max_x
		
	def _init_axes(self, **kw):
		# initialize the axes
		y_step = self.y_step or self.spread_y / 10.0
		
		self.axes.initialize(self._x_map, self._y_map, 0,
					0, self.min_y, self.max_y,
					self.x_box, self.y_box,
					0, y_step)
		
	def _graph(self, **kw):
		## how do we draw the graph itself?
		for graph_entry in self.datalist:
			graph_entry.draw(self._paper, self._x_map, self._y_map, self.bar_width)

	def _x_map(self, x):
		## calculate the actual x value from the given value
		## in this case using the mapping ratio xmap
		if self.reverse_x:
			return  (self.ref_x - self.bar_spacing) - (x * self.xmap)
		else:
			return  (self.ref_x + self.bar_spacing) + (x * self.xmap)
		
class Bar_Entry(Graph_Entry):
	def draw(self, paper, x_map, y_map, width, color=None, line_width=1,
				outline='black', out_width=1):
		if not self.in_graph:
			return
			
		color = color or self.color
		
		x0 = x_map(self.get_x())
		x1 = x0 + width
		y0 = y_map(0)
		y1 = y_map(self.get_y())
				
		self.symbol.draw(paper, x0, y0, x1=x1, y1=y1,
				color=color, line_width=out_width,
				line_color=outline)

class Bar_Axes:
	def __init__(self, x_axis=None, y_axis=None, 
				background=None, foreground=None,
				graph=None, **kw):
		# x axis record
		# self.x_axis = x_axis or X_Axis()
		
		# y axis record
		self.y_axis = y_axis or Y_Bar_Axis()
		
		# background of graph
		self.background = background or Background()
		
		# foreground of graph (i.e. over the data)
		self.foreground = foreground
		
		# parent graph
		self.graph = graph
		
	def initialize(self, x_map_func, y_map_func, 
			xmin, xmax, ymin, ymax, 
			x_box, y_box, x_step, y_step, **kw):
		# self.x_axis.initialize(x_map_func, xmin, xmax, x_box, x_step)
		self.y_axis.initialize(y_map_func, ymin, ymax, y_box, y_step)
		
	def draw(self, paper, **kw):
		# self.x_axis.draw(paper)
		self.y_axis.draw(paper)
		
class Y_Bar_Axis(Y_Axis):
	def draw(self, paper, bound_box=None, color='black', **kw):
		def plot_tick(paper, y, y0, x0, x1, color, font=bold):
			paper.create_line(x0, y0, x1, y0, width=2, fill=color,
					tags=AXIS_Y_LINE_TAG)
			text = get_marker(y)
			paper.create_text(x0, y0, text=text, font=font,
						fill=color, anchor='e',
						tags=AXIS_Y_TEXT_TAG)
						
		bound_box = bound_box or self.y_box
		x0 = bound_box[RGT] - 6
		x1 = bound_box[RGT]
		
		ymin = self.ymin
		ymax = self.ymax
		y_step = self.y_step
		
		stop = 0
		y = 0.0
		while not stop:
			y0 = self.y_map(y)
			plot_tick(paper, y, y0, x0, x1, color)
			y = y + y_step
			if y > ymax:
				stop = 1
		
		y = float(-y_step)		
		while stop:
			if y < ymin:
				stop = 0
			y0 = self.y_map(y)
			plot_tick(paper, y, y0, x0, x1, color)
			y = y - y_step
			
		
## Implementation of a Line Graph class set
class Line_Graph(Graph_Record):
	pass
	
class Line_Entry(Graph_Entry):
	def get_symbol(self):
		return Line_Symbol()

	def draw(self, paper, x_map, y_map, width=None, color=None):
		if not self.in_graph:
			return
			
		color = color or self.color
		width = width or self.s_size
		
		# graph record list takes precedence
		if self.reclist:	
		    for rec in self.reclist:
		    	if self._filter_hook_A(rec):
		    		rec.draw(paper, x_map, y_map, color=color, width=width)
		
		# a list type?	
		elif self.type[0] == 'l':
			# is the data a list of records?
			if self.type == ListRecordType:
				for index in range(1, len(self.data)):
					member0 = self.data[(index-1)]
					member1 = self.data[index]
					if self._filter_hook_B(member0):
					    x0 = x_map(self.get_x(member0))
					    y0 = y_map(self.get_y(member0))
					    x1 = x_map(self.get_x(member1))
					    y1 = y_map(self.get_y(member1))
					    self.symbol.draw(paper, x0, y0, x1, y1,
					    		width=width, color=color)	
			
			elif self.type == ListIndexedType:		
				for index in range(1, len(self.data[self.map_x])):
					x0 = x_map(self.data[self.map_x][(index-1)])
					y0 = y_map(self.data[self.map_y][(index-1)])
					x1 = x_map(self.data[self.map_x][index])
					y1 = y_map(self.data[self.map_y][index])
					# filtB can be used for things like wraparound
					# graphs where the line may have to be broken
					if self._filter_hook_B((x0, y0, x1, y1)):
						self.symbol.draw(paper, x0, y0, x1, y1,
							    width=width, color=color)
			    
class Line_Symbol(Symbol):
	def draw(self, paper, x0, y0, x1, y1, width=None, color='black', tag=''):
		paper.create_line(x0, y0, x1, y1, width=width, fill=color)
			
	def legend(self, paper, x, y, size, color, outline='black', line_width=1, line_color='black'):
		# plot a box for the legend
		plot_box = (x, y, x+size, y+size)
		paper.create_rectangle(plot_box, fill=color, width=line_width,
						outline=outline, tag='legend')


# Test the Graphs Module
if __name__ == '__main__':

	# testing the graphs
	graph_dat = []
	graph_dat.append(Bar_Entry(title='Test #1', color='red', x=0, y=55))
	graph_dat.append(Bar_Entry(title='Goobers', color='blue', x=1, y=10))
	graph_dat.append(Bar_Entry(title='2', color='yellow', x=2, y=40))
	graph_dat.append(Bar_Entry(title='Goobers3', color='orange', x=3, y=30))
	graph_dat.append(Bar_Entry(title='Goobers4', color='brown', x=4, y=90))
	bar_graph = Bar_Graph(datalist=graph_dat, height=0, y_axis_width=20,
			x_axis_width=40, bg_color='turquoise', 
			legend= Legend_Record(bg_color='grey', bg_box_width=3))
	frame = Tk()
	bar_graph._make_graph(frame)
	frame.mainloop()
