#! /usr/bin/env python

#	
#	Report.py	
#	
#	Python module to perform formatting functions
#		for report-like representation of 
#		multi-field data.
#
#	Jason McDermott, JEMWorks Software 7/26/00,
#			 jemworks@jasonya.com
#	
#	

import string
from types import *

HORIZ_RULE = '________________________________________________________________________'

TRUE  = 1
FALSE = 0

FLAG = ('FLAG', 'flag')
DICT = ('DICT', 'dict')

###############################################
# Class: Field Record
####
#	Class for single field representation.
#
####
#	Variables:
#
#	key- 	unique(?) key for referencing data
#	label- 	label for heading columns, etc.
#
#		The following variables refer to the formatting
#		of the field data, in the form %[f_len][f_type]
#		for the print representation (e.g. %10s, %-5i, etc.)
#	f_len- 	format length
#	f_type- format type
#		Note that f_len is intentionally left as a string
#		so that other designators (e.g. -,0, , etc) can
#		be included. Note that the f_type supports a special
#		designation as a flag type variable, on or off, using
#		the syntax f_type='FLAG=[on_string]$[off_string], so for 
#		true/false variable, FLAG=T$F, FLAG=+$-, etc.
#
#	description-	a description of the data that the
#			field contains
#	include-	flag denoting whether or not the
#			field should be included in a report
#
####	
#	Methods:
#	
#	get_value: given a value, returns a string with
#			the formatted representation of
#			the value.
#
#	get_label: returns the label in the appropriate
#			length for a coulumn header
#
#

class Field_Record:
	def __init__(self, key, label, f_len='10', 
			f_type='s', description='',
			include=1):
		self.key = key
		self.label = label
		self.f_len = f_len
		self.f_type = f_type
		self.description = description
		self.include = include
		
		leng = string.split(self.f_len, '.')[0]
		if not leng:
			self.leng = 0
		else:
			s = 0
			for l in leng:
				if l not in string.digits:
					s = s + 1
			leng = leng[s:]		
			self.leng = string.atoi(leng)
		
	def get_value(self, value, simple=0):
		if value == None:
			return string.center('None', (self.leng+1))
		if simple:
			print_str = "\' %%%s\' %% value" % self.f_type
		else:
			print_str = "\' %%%s%s\' %% value" % (self.f_len, self.f_type)
		return eval(print_str)
	
	def get_label(self, center=0):
		if center:
			return string.center(self.label, (self.leng+1))
		print_str = "\' %%%ss\' %% self.label" % self.leng
		return eval(print_str)
		
	def set_include(self, set=1):
		self.include = set

################
# Class: Field_Flag
#
#	This class can be used to represent a plus
#	minus flag variable. Specify the f_type as
#	FLAG=[On Char]$[Off Char], 
#	for example the default will be FLAG=+$-, 
#	can also be used as FLAG=T$F
#
		
class Field_Flag(Field_Record):
	def __init__(self, key, label, f_len='3',
			f_type=FLAG, description='', 
			include=1):
			
		self.key = key
		self.label = label
		self.f_len = f_len
		self.f_type = f_type
		self.f_onchar,self.f_offchar = self.parse_type(f_type)
		self.description = description
		self.include = include
		
		leng = string.split(self.f_len, '.')[0]
		s = 0
		for l in leng:
			if l not in string.digits:
				s = s + 1
		leng = leng[s:]		
		self.leng = string.atoi(leng)
	
	def parse_type(self, f_type):
		if f_type in FLAG:
			return "+","-"
		chars = string.split(f_type, '=')[1]
		chars = string.split(chars, '$')
		return chars[0], chars[1]
			
	def get_value(self, value, **kw):
		if not value:
			print_str = "\'%%%ss\' %% self.f_offchar" % (self.f_len)
			return eval(print_str)
		print_str = "\'%%%ss\' %% self.f_onchar" % (self.f_len)
		return eval(print_str)
		
################
# Class: Field_Dict
#
#	This class can be used to translate
#	a value into FIXME
#
		
class Field_Dict(Field_Record):
	def __init__(self, key, label, f_len='3',
			f_type=DICT, description='', 
			include=1):
			
		self.key = key
		self.label = label
		self.f_len = f_len
		self.f_type = f_type
		self.dict= self.parse_type(f_type)
		self.description = description
		self.include = include
		
		leng = string.split(self.f_len, '.')[0]
		s = 0
		for l in leng:
			if l not in string.digits:
				s = s + 1
		leng = leng[s:]		
		self.leng = string.atoi(leng)
	
	def parse_type(self, f_type):
		index = string.find(f_type, "$")
		if index < 0:
			return
		
		index = index + 1
		dict = {}
			
		f_type = f_type[index:]
		pairs = string.split(f_type, ',')
		for pair in pairs:
			value, key = string.split(pair, ":")
			value = string.atof(value)
			dict[value] = string.strip(key)
		
		return dict
			
	def get_value(self, value, **kw):
		if not self.dict.has_key(value):
			print_str = "\'%%%ss\' %% \'Unkown\'" % (self.f_len)
		else:
			print_str = "\'%%%ss\' %% self.dict[value]" % (self.f_len)
		return eval(print_str)

###############################################
# Class: Report Record
####
#	Class for multiple-field representation.
#
#
#
#
#
#

class Report_Record:
	def __init__(self, header=None, description='',
			field_list=[]):
		self.header = header
		self.description = description
		self.field_list = field_list
		
	######
	# Routine: init_list
	# takes a list of tuples in the form:
	# (key, label, f_len, f_type) -or-
	# (key, label, f_len, f_type, include) -or-
	# (key, label, f_len, f_type, include, description)
	#
	# and transforms it into the filed_record list
	
	def init_list(self, list):
		field_list = []
		if len(list[0]) == 4:
		    for (key,label,f_len,f_type) in list:
		    	field_list.append(self.get_field(key=key, label=label,
					f_len=f_len, f_type=f_type,
					include=1, description=''))
			
		if len(list[0]) == 5:
		    for (key,label,f_len,f_type,include) in list:
			field_list.append(self.get_field(key=key, label=label,
					f_len=f_len, f_type=f_type,
					include=include, description=''))
					    
		elif len(list[0]) == 6:
		    for (key,label,f_len,f_type,include,description) in list:
			field_list.append(self.get_field(key=key, label=label,
					f_len=f_len, f_type=f_type,
					include=include, 
					description=description))	
		
		# Warning- bombs existing list, if any			
		self.field_list = field_list
	
	# parsing subroutine for init_list
	def get_field(self, key=None, label=None,
			f_len=None, f_type=None, include=None, 
			description=None):
			
		if f_type[0:4] in FLAG:
			return Field_Flag(key, label,
				f_len=f_len, f_type=f_type,
				include=include, description=description)

		if f_type[0:4] in DICT:
			return Field_Dict(key, label,
				f_len=f_len, f_type=f_type,
				include=include, description=description)
					
		return Field_Record(key, label, f_len=f_len, f_type=f_type,
				description=description, include=include)
	
	####################################
	# Subroutine: column_get_header
	####
	#	Prints a header for column type output of variables
	####
	#	Variables:
	#
	#	header-		boolean, print out header
	#	legend-		boolean, print column type 
	#				legend (i.e. column headers)
	#	hr-		boolean, add horizontal rule
	#	center-		boolean, center column headers
	#	description-	boolean, print description
	#
	#
				
	def column_get_header(self, header=1, legend=1, hr=1, 
				description=0, center=0, label=0, order=None):
		report = ''
		if not label:
		    if header:
			    report = "%s\n" % self.header
		    if description:
			    report = "%sDescription:\n%s\n\n" % (report, self.description)
		if legend or label:
			if not order:
				for field in self.field_list:
					if field.include:
						report = "%s%s" % (report, field.get_label(center))
			else:
				if type(order) not in (ListType, TupleType):
					print "Report Error: order key not list type"
				for n in order:
					try:
						field = self.field_list[(n-1)]
					except IndexError:
						print ("Report Error: order number %d out of range 1-%d, skipped"
								% (n, len(self.field_list)))
						continue
					report = "%s%s" % (report, field.get_label(center))
			if label:
				report = report + "\n"
					
		if not label and hr:
			report = "\n%s\n%s\n" % (report, HORIZ_RULE)
			
		return report
	
	##################################
	# Subroutine: objects_get_row
	####
	# 	Given a number of objects (with __dict__'s) attempts
	#	to match format keys with object attributes and 
	#	return a row in column format. See __main__, below,
	#	for test example.
	####
	#
	#	Arguments:
	#	
	#	an object or objects with attributes (i.e. a __dict__)
	#
		
	def objects_get_row(self, *args, **kw):
		if kw.has_key('order'):
			order = kw['order']
		else:
			order = None
		
		# if no order list is given use the field.include flags
		if not order:
			row = ''
			for field in self.field_list:
			    if field.include:
				found = 0
				for object in args:
				    if object.__dict__.has_key(field.key):
					value = object.__dict__[field.key]
					row = "%s%s" % (row, field.get_value(value))
					found = 1
					break
				if not found:
					print ("Report Error: field %s not found in objects" %
						field.key)
			return row
		
		if type(order) not in (ListType, TupleType):
			print "Report Error: order key is not a list"
			return
				
		row = ''
		for n in order:
			try:
				field = self.field_list[(n-1)]
			except IndexError:
				print ("Report Error: order number %d out of range 1-%d, skipped"
						% (n, len(self.field_list)))
				continue
				
			found = 0
			for object in args:
			    if object.__dict__.has_key(field.key):
				value = object.__dict__[field.key]
				row = "%s%s" % (row, field.get_value(value))
				found = 1
				break
			if not found:
				print ("Report Error: field %s not found in objects" %
					field.key)
		return row

	##################################
	# Subroutine: objects_get_db
	####
	# 	Given a number of objects (with __dict__'s) attempts
	#	to match format keys with object attributes and 
	#	return the contents of a Report_Record.field_list in 
	#	report database (db) format.
	####
	#
	#	Arguments:
	#	
	#	an object or objects with attributes (i.e. a __dict__)
	#	
	def objects_get_db(self, *args, **kw):
		db = ''
		for field in self.field_list:
		    if field.include:
			found = 0
			for object in args:
			    if object.__dict__.has_key(field.key):
				value = object.__dict__[field.key]
				db = "%s%-20s:%15s: %s\n" % (db, field.label, 
						field.key, field.get_value(value, simple=1))
				found = 1
				break
			if not found:
				print ("Report Error: field %s not found in objects" %
					field.key)
		return db

	##################################
	# Subroutine: parse_db
	####
	# 	Given a number of objects (with __dict__'s) attempts
	#	to match format keys with object attributes and 
	#	return the contents of a Report_Record.field_list in 
	#	report database (db) format.
	####
	#
	#	Arguments:
	#	
	#	an object or objects with attributes (i.e. a __dict__)
	#
	def parse_db(self, object=None, lines=None, index=None, fast=FALSE, **kw):
		stop = FALSE
		
		while not stop:
			try:
				tokens = string.split(lines[index], ':')
			except IndexError:
				stop = TRUE
				index = -1
			else:
				if len(tokens) > 2:	
				    key = string.strip(tokens[1])
				    value = tokens[2]
				    if fast:
					try:
					    # integer value
					    object.__dict__[key] = string.atoi(value)
					except ValueError:
					    try:
						# float value
						object.__dict__[key] = string.atof(value)
					    except ValueError:
						# string value
						object.__dict__[key] = string.strip(value)
				    else:
					    found = FALSE
					    for field in self.field_list:
						    if field.key == key:
							    if field.f_type == 'i':
								    object.__dict__[key] = string.atoi(value)
							    elif field.f_type == 'f':
								    object.__dict__[key] = string.atof(value)
							    else:
								    object.__dict__[key] = string.strip(value)
							    found = TRUE
					    if not found:
						    print 'ReportError: Key %s not found in object' % key
								    
				# increment the line index
				index = index + 1
			
		return index

	def get_include_list(self):
		list = []
		for field in self.field_list:
			list.append(field.include)
		return list
	
	def set_include_list(self, list, ordered=0):
		if not ordered:
			for i in range(0, len(list)):
				self.field_list[i].include = list[i]
			return
			
	def set_include_dict(self, dict):
		for member in self.field_list:
			if dict.has_key(member.key):
				member.include = dict[member.key]
			else:
				member.include = 0
		
#########################################
#
#	Test Run
#
####
TEST_TUPLES =  (('color', 'COLOR', '17', 's'),		# 1
		('floater', 'FLOAT', '10.2', 'f'),	# 2
		('inter', 'INTEGER', '10', 'i'),	# 3
		('chrom', 'CHROM', '3', 'FLAG=T$F'),	# 4
		('type', 'CATEGORY', '15', 's'))	# 5
		
ORDER = (4,2,3,1)
		
class Test_Object:
	def __init__(self, color, floater, inter, chrom):
		self.color = color
		self.floater = floater
		self.inter = inter
		self.chrom = chrom
		self.type = 1
		
class Category:
	def __init__(self, type):
		self.type = type
		
TEST_VALUES = (('red', 15.4, 10, 1), ('blue', 123.4, 100, 0),
		('chartreuse', -0.09, 1, 1), ('bluish', 222.2, 2, 1))
		
if __name__ == '__main__':
	list = []
	category = Category('Colorifics')
	for tup in TEST_VALUES:
		list.append(Test_Object(tup[0], tup[1], tup[2], tup[3]))
	form = Report_Record(header='Test #1', 
		description='A test of Report.py module')
	form.init_list(TEST_TUPLES)
	
	print "Report Output Ordered as Format Definition\n"
	print form.column_get_header(1,1,1,1,0,1)
	for test in list:
		# NOTE: for getting rows the objects_get_rows
		#	searches the arg list sequentially,
		#	meaning that the attribute 'type'
		# 	(which is in both objects) will be
		#	used from the category object.
		print form.objects_get_row(category, test)
	
	print "\n***************************************************************\nReport Output Ordered as ORDER Definition\n"
	print form.column_get_header(1,1,1,1,0,1, order=ORDER)
	for test in list:
		print form.objects_get_row(category, test, order=ORDER)		
		
		
			
		
			
		
				
				
			
	
	
	
